using System.Collections.Generic;
using Nuwanda.Common;
using System.ComponentModel;
using System;
using Microsoft.Practices.EnterpriseLibrary.Validation;

namespace Nuwanda.TfsSpotlight.Shell.Data
{
    /// <summary>A base class for all entities (provides IsDirty mechanism and implements INotifyPropertyChanged interface)</summary>
    [Author("Bartomiej Legid")]
    [Serializable]
    public class BaseEntity<T> : IBaseEntity where T : BaseEntity<T>
    {
        [NonSerialized]
        private bool _isDirty;
        /// <summary>Checks whether the entity has been modified</summary>
        public virtual bool IsDirty
        {
            get { return this._isDirty;  }
        }

        [NonSerialized]
        private bool _isValid;
        public virtual bool IsValid
        {
            get { return this._isValid; }
        }

        public string Error
        {
            get
            {
                return this._isValid ? String.Empty : "Encja nie jest poprawna";
            }
        }

        private readonly Dictionary<string, string> _errors;

        public string this[string columnName]
        {
            get
            {
                if (this._errors.ContainsKey(columnName))
                    return this._errors[columnName];
                
                return String.Empty;
            }
        }

        protected BaseEntity()
        {
            this._isDirty = false;
            this._isValid = true;
            this._errors = new Dictionary<string, string>();
        }

        /// <summary>Ocurres when a property of the entity changes</summary>
        public event PropertyChangedEventHandler PropertyChanged = delegate { };

        /// <summary>Invoke this method when changing a property</summary>
        /// <param name="propertyName">Name of changed property</param>
        protected virtual void OnPropertyChanged(string propertyName)
        {
            this._isDirty = true;
            Validate();
            this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        protected void Validate()
        {
            var results = Validation.Validate(this as T);
            this._isValid = results.IsValid;

            if(results.IsValid)
            {
                this._errors.Clear();
            }
            else
            {
                foreach (var result in results)
                {
                    if (!this._errors.ContainsKey(result.Key))
                        this._errors.Add(result.Key, result.Message);
                }
            }
        }

        /// <summary>Marks the entity as old (= not dirty)</summary>
        public virtual void MarkOld()
        {
            this._isDirty = false;
        }
    }
}