using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using Microsoft.TeamFoundation;
using Microsoft.TeamFoundation.Build.Proxy;
using Microsoft.TeamFoundation.VersionControl.Client;
using Microsoft.TeamFoundation.WorkItemTracking.Client;
using Microsoft.TeamFoundation.WorkItemTracking.Controls;
using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Data;
using Nuwanda.TfsSpotlight.Services.Interfaces;
using Nuwanda.TfsSpotlight.Shell.Services;
using Nuwanda.TfsSpotlight.Shell.Services.Interfaces;
using StructureMap;
using ILogger=Nuwanda.TfsSpotlight.Shell.Services.Interfaces.ILogger;

namespace Nuwanda.TfsSpotlight.TFS
{
    [Author("Bartomiej Legid")]
    [Pluggable("TfsServerManager")]
    public class TfsServerManager : ITfsServerManager
    {
        protected ITeamFoundationServer _server;

        public ServerDescriptor ServerDescriptor { get; set; }

        private ITeamFoundationServerFactory TeamFoundationServerFactory { get; set; }
        private Microsoft.TeamFoundation.Client.ICredentialsProvider CredentialsProvider { get; set; }
        private IMonitService MonitService { get; set; }
        private ILogger Logger { get; set; }
        private IWorker Worker { get; set; }
        private IHistoryService HistoryService { get; set; }

        #region Events

        public event EventHandler<ErrorEventArgs> Error = delegate { };

        public event EventHandler Connecting = delegate { };
        public event EventHandler Connected = delegate { };

        public event EventHandler Reconnecting = delegate { };
        public event EventHandler Reconnected = delegate { };
        
        public event EventHandler Disconnecting = delegate { };
        public event EventHandler Disconnected = delegate { };

        public event EventHandler ConnectionLost = delegate { };
        

        public event EventHandler RetrievingProjects = delegate { };
        public event EventHandler ProjectsRetrieved = delegate { };
        #endregion

        public TfsServerManager(ITeamFoundationServerFactory teamFouncationServerFactory,
            Microsoft.TeamFoundation.Client.ICredentialsProvider credentialsProvider,
            IMonitService monitService, ILogger logger, IWorker worker, IHistoryService historyService)
        {
            this.TeamFoundationServerFactory = teamFouncationServerFactory;
            this.CredentialsProvider = credentialsProvider;
            this.MonitService = monitService;
            this.Logger = logger;
            this.Worker = worker;
            this.HistoryService = historyService;
        }

        private bool _isConnectionLost;
        public bool IsConnectionLost
        {
            get { return _isConnectionLost; }
            private set
            {
                _isConnectionLost = value;
                if (_isConnectionLost)
                {
                    ConnectionLost(this, EventArgs.Empty);
                }
            }
        }

        public bool IsConnected
        {
            get { return this._server != null && this._server.HasAuthenticated; }
        }

        public bool IsWorking
        {
            get { return this.Worker.IsWorking; }
        }

        /// <exception cref="InvalidOperationException">Thrown when the server was not set up properely.</exception>
        public void Connect()
        {
            if (this.ServerDescriptor == null)
                throw new InvalidOperationException("You must first set a proper ServerDescriptor");

            if (this.Worker.BeginWork() == false)
                return;

            this.Connecting(this, EventArgs.Empty);
            
            Exception exception = null;
            var errorMessage = String.Empty;
            try
            {
                if (this._server != null && this._server.HasAuthenticated)
                    return;

                if (this._server == null)
                {
                    this._server = this.TeamFoundationServerFactory.GetServer(
                        this.ServerDescriptor.Url, this.CredentialsProvider);
                }

                this._server.Authenticate();
            }
            catch (WebException we)
            {
                exception = we;
                errorMessage = we.Message;
            }
            catch (TeamFoundationServerUnauthorizedException te)
            {
                exception = te;
                errorMessage = String.Format(Messages.MsgNotAuthorizedToConnect, this.ServerDescriptor.Url);
            }
            catch (TeamFoundationInvalidServerNameException isne)
            {
                exception = isne;
                errorMessage = String.Format(Messages.MsgInvalidTfsServerName, this.ServerDescriptor.Url);
            }
            catch (Exception e)
            {
                exception = e;
                errorMessage = String.Format(Messages.MsgInternalError, e.Message);
            }
            finally
            {
                this.Worker.EndWork();
                if (exception != null)
                {
                    this.Error(this, new ErrorEventArgs(errorMessage, exception));
                    this.Logger.WriteException(exception);
                }
                else
                {
                    this.IsConnectionLost = false;
                    this.Connected(this, EventArgs.Empty);
                }
            }
        }

        /// <exception cref="InvalidOperationException">Thrown when there is no server to reconect to.</exception>
        public void Reconnect()
        {
            if (this._server == null)
                throw new InvalidOperationException("Server was never connected.");

            if (this.Worker.BeginWork() == false)
                return;

            this.Reconnecting(this, EventArgs.Empty);

            Exception exception = null;
            var errorMessage = String.Empty;
            try
            {
                this._server.Authenticate();
            }
            catch (WebException we)
            {
                exception = we;
                errorMessage = we.Message;
            }
            catch (TeamFoundationServerUnauthorizedException te)
            {
                exception = te;
                errorMessage = String.Format(Messages.MsgNotAuthorizedToConnect, this.ServerDescriptor.Url);
            }
            catch (TeamFoundationInvalidServerNameException isne)
            {
                exception = isne;
                errorMessage = String.Format(Messages.MsgInvalidTfsServerName, this.ServerDescriptor.Url);
            }
            catch (Exception e)
            {
                exception = e;
                errorMessage = String.Format(Messages.MsgInternalError, e.Message);
            }
            finally
            {
                this.Worker.EndWork();
                if (exception != null)
                {
                    this.Error(this, new ErrorEventArgs(errorMessage, exception));
                    this.Logger.WriteException(exception);
                }
                else
                {
                    this.IsConnectionLost = false;
                    this.Reconnected(this, EventArgs.Empty);
                }
            }
        }

        public void Disconnect()
        {
            if (this._server == null) return;

            this.Disconnecting(this, EventArgs.Empty);

            this._server.Dispose();
            this._server = null;
            this.ServerDescriptor.Projects.Clear();

            this.IsConnectionLost = false;
            this.Disconnected(this, EventArgs.Empty);
        }

        public List<ProjectDescriptor> GetProjects()
        {
            if (this.Worker.BeginWork() == false)
                return null;

            this.RetrievingProjects(this, EventArgs.Empty);

            if(this.IsConnected == false)
            {
                this.Error(this, new ErrorEventArgs(Messages.MsgNotConnected, null));
            }
                
            var projects = new List<ProjectDescriptor>();

            try
            {
                var store = this._server.GetWorkItemStore();
                if (store == null) return projects;

                var tfsProjects = store.Projects.Cast<Project>();
                foreach (var tfsProject in tfsProjects)
                {
                    if (!tfsProject.HasWorkItemReadRights) continue;

                    var project = new ProjectDescriptor(tfsProject.Name, this.ServerDescriptor);
                    project.Queries.AddRange(
                        from q in tfsProject.StoredQueries.Cast<StoredQuery>().AsQueryable()
                        select new QueryDescriptor(q.Name, q.QueryGuid, q.QueryText, q.QueryScope == QueryScope.Private, project));

                    project.WorkItemTypes = new List<WorkItemType>(tfsProject.WorkItemTypes.Cast<WorkItemType>());
                    project.BuildTypes = GetBuildTypes(project);
                    projects.Add(project);
                }

                this.ServerDescriptor.Projects.Clear();
                this.ServerDescriptor.Projects.AddRange(projects);
                this.ProjectsRetrieved(this, EventArgs.Empty);

                return projects;
            }
            catch (Exception e)
            {
                this.Logger.WriteException(e);
                this.Error(this, new ErrorEventArgs(String.Format(Messages.MsgInternalError, e.Message), e));
                return projects;
            }
            finally
            {
                this.Worker.EndWork();
            }
        }

        private List<BuildDescriptor> GetBuildTypes(ProjectDescriptor project)
        {
            var vcs = this._server.GetVersionControlServer();
            var itemSet = vcs.GetItems("$/" + project.Name + "/TeamBuildTypes", VersionSpec.Latest, RecursionType.OneLevel, DeletedState.NonDeleted, ItemType.Folder);
            var buildTypes = new List<BuildDescriptor>();

            foreach (var item in itemSet.Items)
            {
                var buildType = Path.GetFileName(item.ServerItem);
                if (buildType != "TeamBuildTypes")
                {
                    buildTypes.Add(new BuildDescriptor(project, buildType));
                }
            }

            return buildTypes;
        }

        public List<BuildDataDescriptor> GetBuilds(BuildDescriptor buildDescriptor)
        {
            if (this.Worker.BeginWork() == false)
                return null;

            if (this.IsConnected == false)
            {
                this.Error(this, new ErrorEventArgs(Messages.MsgNotConnected, null));
                return null;
            }

            if (buildDescriptor == null) return null;

            try
            {
                var buildDataList = new List<BuildData>(this._server.GetBuildStore().GetListOfBuilds(buildDescriptor.Project.Name, buildDescriptor.BuildType));
                buildDataList.Reverse();
                return buildDataList.ConvertAll(data => new BuildDataDescriptor(data, this._server.UserName));

            }
            catch(WebException we)
            {
                this.IsConnectionLost = true;
                this.Logger.WriteException(we);
                throw new CouldNotConnectToTfsServerException(String.Format(Messages.ExConnectionError, this.ServerDescriptor.Url), we);
            }
            catch (ConnectionException ce)
            {
                this.IsConnectionLost = true;
                this.Logger.WriteException(ce);
                throw new CouldNotConnectToTfsServerException(String.Format(Messages.ExConnectionError, this.ServerDescriptor.Url), ce);
            }
            catch (Exception e)
            {
                this.Logger.WriteException(e);
                throw;
            }
            finally
            {
                this.Worker.EndWork();
            }
        }

        public QueryResult RunQuery(QueryDescriptor query)
        {
            if (this.Worker.BeginWork() == false)
                return null;

            if (this.IsConnected == false)
            {
                this.Error(this, new ErrorEventArgs(Messages.MsgNotConnected, null));
                return null;
            }

            if (query == null) return null;

            try
            {
                this.MonitService.ShowStatusMessage(String.Format(Messages.MsgRunningQuery, query.Name), MessageSeverity.Retrieving);

                var store = this._server.GetWorkItemStore();
                if (store == null) return null;

                var context = new Dictionary<string, string>();
                context.Add("project", query.Project.Name);

                var result = store.Query(query.Text, context);
                this.MonitService.ClearStatusMessage();

                return new QueryResult(result, new ResultOptions(store.FieldDefinitions, query.Text));
            }
            catch (ConnectionException ce)
            {
                this.IsConnectionLost = true;
                this.Logger.WriteException(ce);
                throw new CouldNotConnectToTfsServerException(String.Format(Messages.ExConnectionError, this.ServerDescriptor.Url), ce);
            }
            catch (ValidationException ve)
            {
                this.Logger.WriteException(ve);
                throw new QueryValidationException(String.Format(Messages.ExInvalidQueryDefinition, ve.Message), ve);
            }
            catch (Exception e)
            {
                this.Logger.WriteException(e);
                throw;
            }
            finally
            {
                this.Worker.EndWork();
            }
        }

        public void SaveWorkItems(List<WorkItem> workItems)
        {
            if (this.Worker.BeginWork() == false)
                return;

            if (workItems == null || workItems.Count == 0) return;

            if (this.IsConnected == false)
            {
                this.Error(this, new ErrorEventArgs(Messages.MsgNotConnected, null));
            }

            try
            {
                this.MonitService.ShowStatusMessage(Messages.MsgSavingWorkItemBatch, MessageSeverity.Saving);

                var store = this._server.GetWorkItemStore();
                if (store == null) return;

                var errors = store.BatchSave(workItems.ToArray());

                if (errors.Length > 0)
                {
                    this.MonitService.ShowStatusMessage(
                        String.Format(Messages.MsgBatchSaveError, errors[0].Exception.Message),
                        MessageSeverity.Error);
                }
                else
                {
                    this.MonitService.ShowStatusMessage(Messages.MsgSaved, MessageSeverity.Information);
                }
            }
            catch(ConnectionException ce)
            {
                this.IsConnectionLost = true;
                this.Logger.WriteException(ce);
                throw new CouldNotConnectToTfsServerException(String.Format(Messages.ExConnectionError, this.ServerDescriptor.Url), ce);
            }
            catch (Exception e)
            {
                this.Logger.WriteException(e);
                throw;
            }
            finally
            {
                this.Worker.EndWork();
            }
        }

        /// <exception cref="ConcurrentAccessException">Thrown when there was a concurrent access to the saved work item.</exception>
        /// <exception cref="WorkItemValidationException">Thrown when the saved work item is not valid.</exception>
        /// <exception cref="CouldNotConnectToTfsServerException">Thrown when there was an error with server connection.</exception>
        public void SaveWorkItem(WorkItem workItem)
        {
            if (this.Worker.BeginWork() == false)
                return;

            if (workItem == null) return;

            if (this.IsConnected == false)
            {
                this.Error(this, new ErrorEventArgs(Messages.MsgNotConnected, null));
            }

            try
            {
                this.MonitService.ShowStatusMessage(String.Format(Messages.MsgSavingWorkItem, workItem.Title), MessageSeverity.Saving);

                bool isNew = workItem.Id > 0;

                workItem.Save();

                this.HistoryService.RegisterOperation(new HistoryEntry(workItem.Id, workItem.Title, this.ServerDescriptor.Url, isNew ? HistoryOperationType.Add : HistoryOperationType.Edit));
                this.MonitService.ShowStatusMessage(Messages.MsgSaved, MessageSeverity.Information);
            }
            catch(ConnectionException ce)
            {
                this.IsConnectionLost = true;
                this.Logger.WriteException(ce);
                throw new CouldNotConnectToTfsServerException(String.Format(Messages.ExConnectionError, this.ServerDescriptor.Url), ce);
            }
            catch(ItemAlreadyUpdatedOnServerException iaue)
            {
                this.Logger.WriteException(iaue);
                throw new ConcurrentAccessException(Messages.ExItemAlreadyUpdated, iaue);
            }
            catch (ValidationException ve)
            {
                this.Logger.WriteException(ve);
                throw new WorkItemValidationException(String.Format(Messages.ExInvalidWorkItemDefinition, ve.Message), ve);

            }
            catch(Exception e)
            {
                this.Logger.WriteException(e);
            }
            finally
            {
                this.Worker.EndWork();
            }
        }

        public WorkItem GetWorkItem(int workItemId)
        {
            if (this.Worker.BeginWork() == false)
                return null;

            try
            {
                this.Logger.WriteActivity(String.Format(Messages.MsgLoadingWorkItem, workItemId, this.ServerDescriptor.Name));

                var store = this._server.GetWorkItemStore();
                return store == null ? null : store.GetWorkItem(workItemId);
            }
            catch(DeniedOrNotExistException nee)
            {
                this.Logger.WriteException(nee);
                throw new NotExistsException(String.Format(Messages.MsgWorkItemNotFound, workItemId, this.ServerDescriptor.Name));
            }
            catch (ConnectionException ce)
            {
                this.IsConnectionLost = true;
                this.Logger.WriteException(ce);
                throw new CouldNotConnectToTfsServerException(String.Format(Messages.ExConnectionError, this.ServerDescriptor.Url), ce);
            }
            catch (Exception e)
            {
                this.Logger.WriteException(e);
                throw;
            }
            finally
            {
                this.Worker.EndWork();
            }
        }

        public void Refresh(WorkItem workItem)
        {
            if (workItem == null) return;

            if (this.Worker.BeginWork() == false) return;

            try
            {
                this.Logger.WriteActivity(String.Format(Messages.MsgRefreshingWorkItem, workItem.Title));
                workItem.SyncToLatest();
                
            }
            catch (DeniedOrNotExistException nee)
            {
                this.Logger.WriteException(nee);
                throw new NotExistsException(String.Format(Messages.MsgWorkItemNotFound, workItem.Id, this.ServerDescriptor.Name));
            }
            catch (ConnectionException ce)
            {
                this.IsConnectionLost = true;
                this.Logger.WriteException(ce);
                throw new CouldNotConnectToTfsServerException(String.Format(Messages.ExConnectionError, this.ServerDescriptor.Url), ce);
            }
            catch (Exception e)
            {
                this.Logger.WriteException(e);
                throw;
            }
            finally
            {
                this.Worker.EndWork();
            }
        }

        public void ReportConnectionLost()
        {
            this.IsConnectionLost = true;
        }
    }
}