using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms.DataVisualization.Charting;
using Microsoft.TeamFoundation.WorkItemTracking.Client;
using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Data;
using Nuwanda.TfsSpotlight.Shell.Services.Interfaces;
using Nuwanda.TfsSpotlight.Views.ChartView.ChartData;
using StructureMap;

namespace Nuwanda.TfsSpotlight.Views.ChartView
{
    [Author("Bartomiej Legid")]
    [Pluggable("ChartModel")]
    public class ChartModel : IChartModel
    {
        private IIsolatedStorageService IsolatedStorageService { get; set; }
        private QueryResult QueryResult { get; set; }
        private Series ChartData { get; set; }
        public ChartArea ChartArea { get; private set; }
        public QueryDescriptor Query { get; private set; }
        public List<string> Fields { get; private set; }
        public List<string> ChartTypes { get; private set; }
        public ChartOptions ChartOptions { get; private set; }

        public Title ChartTitle
        {
            get { return new Title(String.Format("'{0}' query result grouped by '{1}' field", this.Query.Name, this.ChartOptions.FieldName), Docking.Top, new Font("Verdana", 20.0f, FontStyle.Bold), Color.Black); }
        }

        public string Title
        {
            get { return String.Format("Chart based on '{0}' query.", this.Query.Name); }
        }

        public string Caption
        {
            get { return String.Format("Chart ({0})", this.Query.Name); }
        }

        public ChartModel(IIsolatedStorageService isolatedStorageService)
        {
            IsolatedStorageService = isolatedStorageService;
            this.ChartOptions = new ChartOptions();
            SetUpChartArea();
            SetUpChartTypes();
        }

        private void SetUpChartArea()
        {
            this.ChartArea = new ChartArea();
            this.ChartArea.Name = "QueryChartArea";
            this.ChartArea.AxisX.IsLabelAutoFit = true;
            this.ChartArea.AxisX.LabelAutoFitStyle = LabelAutoFitStyles.DecreaseFont | LabelAutoFitStyles.IncreaseFont | LabelAutoFitStyles.WordWrap | LabelAutoFitStyles.LabelsAngleStep90;
            this.ChartArea.AxisX.Enabled = AxisEnabled.False;
        }

        public void Load(QueryDescriptor query, QueryResult queryResult)
        {
            this.Query = query;
            this.QueryResult = queryResult;

            LoadConfiguration();

            SetUpWorkItemFields();
        }

        private void SetUpChartTypes()
        {
            this.ChartTypes = new List<string>();
            this.ChartTypes.Add(SeriesChartType.Pie.ToString());
            this.ChartTypes.Add(SeriesChartType.Doughnut.ToString());
            this.ChartTypes.Add(SeriesChartType.Bar.ToString());
            this.ChartTypes.Add(SeriesChartType.Column.ToString());
        }

        public void SetUpWorkItemFields()
        {
            var fields = new List<string>(); 

            if (this.QueryResult.Result.Count > 0)
            {
                var workItem = this.QueryResult.Result[0];
                foreach (Field field in workItem.Fields)
                {
                    fields.Add(field.Name);
                }
            }

            fields.Sort();

            this.Fields = new List<string>();
            this.Fields.AddRange(this.ChartOptions.RecentFieldsSelection);
            this.Fields.Add(String.Empty);
            this.Fields.AddRange(fields);
        }

        private void LoadConfiguration()
        {
            this.ChartOptions = this.IsolatedStorageService.Load<ChartOptions>(this.Query.GetChartConfigurationFileName());
            if(this.ChartOptions == null)
                this.ChartOptions = new ChartOptions();

            this.ChartArea.Area3DStyle.Enable3D = this.ChartOptions.ShowIn3D;
        }

        public void SaveConfiguration()
        {
            this.IsolatedStorageService.Save(this.ChartOptions, this.Query.GetChartConfigurationFileName());
        }

        public Series GenerateChartData()
        {
            var fieldName = this.ChartOptions.FieldName;
            if(String.IsNullOrEmpty(fieldName)) return null;

            var chartData = new StringChartData();
            chartData.CreateData(this.QueryResult, fieldName);

            var series = chartData.GetData(this.ChartOptions);
            series.ChartArea = this.ChartArea.Name;

            this.ChartData = series;

            AssignAlternatingColor();

            return series;
        }

        public void UpdateChartStyle()
        {
            if(this.ChartData == null) return;

            this.ChartArea.Area3DStyle.Enable3D = this.ChartOptions.ShowIn3D;
            this.ChartData.ChartType = this.ChartOptions.ChartType;

            AssignAlternatingColor();
        }

        private void AssignAlternatingColor()
        {
            if (!this.ChartOptions.AssignAlternatingColors) return;

            var index = 0;
            foreach (var dataPoint in this.ChartData.Points)
            {
                if (index % 2 == 1)
                {
                    dataPoint.Color = Color.Orange;
                }
                else
                {
                    dataPoint.Color = Color.LightSkyBlue;
                }

                index++;
            }
        }
    }
}