using System;
using System.Drawing.Imaging;
using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Data;
using Nuwanda.TfsSpotlight.Shell.Proxies.Interfaces;
using Nuwanda.TfsSpotlight.Shell.Services;
using Nuwanda.TfsSpotlight.Shell.Services.Interfaces;
using StructureMap;

namespace Nuwanda.TfsSpotlight.Views.ChartView
{
    [Author("Bartomiej Legid")]
    [Pluggable("ChartPresenter")]
    public class ChartPresenter : TabPresenterBase<IChartView>, IChartPresenter, IServerAssociation
    {
        private IChartModel Model { get; set; }
        private IClipboardService ClipboardService { get; set; }
        private IDialogService DialogService { get; set; }
        private IAsyncService AsyncService { get; set; }
        private IMonitService MonitService { get; set; }

        public ChartPresenter(IChartModel model, IChartView view, IWorker worker, 
            IMenuService menuService, ILogger logger, IClipboardService clipboardService,
            IDialogService dialogService, IAsyncService asyncService, IMonitService monitService) : base(view, worker, menuService, logger)
        {
            this.Model = model;
            this.View.Presenter = this;
            this.ClipboardService = clipboardService;
            this.DialogService = dialogService;
            AsyncService = asyncService;
            MonitService = monitService;
        }

        private bool _initializing;
        public void Initialize()
        {
            this._initializing = true;
            this.View.SetChartArea(this.Model.ChartArea);
            this.View.SetWorkItemFields(this.Model.Fields);
            this.View.SetChartTypesDataSource(this.Model.ChartTypes);
            this.View.SetChartOptionsDataSource(this.Model.ChartOptions);
            this.View.SetHeaderTitle(this.Model.Title);
            this.View.SetHeaderImage(Properties.Resources.Chart);

            this.View.SelectChartType(this.Model.ChartOptions.ChartTypeView);

            this._initializing = false;
        }

        public override void HandleLoad()
        {
            this.Tab.RefreshCaption();
            UpdateChart();
        }

        public override void HandleClose()
        {
            this.Model.SaveConfiguration();
        }

        public override bool IsEqualTo(ITabPresenter presenter)
        {
            if(presenter is ChartPresenter)
            {
                var chartPresenter = presenter as ChartPresenter;
                return chartPresenter.Model.Query.Equals(this.Model.Query);
            }

            return false;
        }

        public override string GetCaption()
        {
            return this.Model.Caption;
        }

        public void HandleCopyChartToClipboard()
        {
            try
            {
                var image = this.View.GetImage(ImageFormat.Bmp);
                this.ClipboardService.SetImage(image);
            }
            catch (InvalidOperationException)
            {
                this.MonitService.ShowStatusMessage(Messages.MsgErrorWhilePuttingDataToTheClipboard, MessageSeverity.Error);
            }
        }

        public void HandleSaveChartAsImage()
        {
            this.DialogService.ConfigureSaveFileDialogBox()
                .Filter("Bitmap (*.bmp)|*.bmp|JPEG (*.jpg)|*.jpg|EMF (*.emf)|*.emf|PNG (*.png)|*.png|GIF (*.gif)|*.gif|TIFF (*.tif)|*.tif", 2)
                .RestoreDirectory();

            var fileName = this.DialogService.GetFileNameForSave();
            var imageFormat = new ImageFormatDetector().Detect(fileName);
            var image = this.View.GetImage(imageFormat);
            image.Save(fileName);
        }

        private bool _updating;
        public void UpdateChart()
        {
            if (this._initializing) return;
            if (this._updating) return;

            this._updating = true;

            this.AsyncService.QueueInPool(
                () => {
                        this.View.DisableControlPanel();
                        this.View.ClearChart();
                        this.View.ShowProgress();

                        this.Model.SetUpWorkItemFields();
                        this.View.SetWorkItemFields(this.Model.Fields);

                        var series = this.Model.GenerateChartData();
                        if (series != null)
                        {
                            this.View.DisplayChart(series);
                            this.View.SetChartTitle(this.Model.ChartTitle);
                        }

                        this.View.HideProgress();
                        this.View.EnableControlPanel();
                        this._updating = false;
                    });
            
        }

        public void UpdateChartStyle()
        {
            if (this._initializing) return;
            if (this._updating) return;

            this.Model.UpdateChartStyle();
        }

        #region Implementation of IServerAssociation

        public bool IsAssociatedWith(ServerDescriptor descriptor)
        {
            return this.Model.Query.Project.Server.Equals(descriptor);
        }
        #endregion
    }
}