using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.TeamFoundation.WorkItemTracking.Client;
using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Data;
using Nuwanda.TfsSpotlight.Services.Interfaces;
using Nuwanda.TfsSpotlight.Shell.Services.Interfaces;
using Nuwanda.TfsSpotlight.TFS;
using Nuwanda.TfsSpotlight.Views.WorkItems;
using StructureMap;

namespace Nuwanda.TfsSpotlight.Views.CreateWorkItem
{
    [Author("Bartomiej Legid")]
    [Pluggable("CreateWorkItemPresenter")]
    public class CreateWorkItemPresenter : ICreateWorkItemPresenter
    {
        private readonly ICreateWorkItemView _view;

        private ITfsServerRepository TfsServerRepository { get; set; }
        private ITabsService TabsService { get; set; }
        private IUserSettingsService UserSettingsService { get; set; }

        private ProjectDescriptor Project { get; set; }

        public CreateWorkItemPresenter(ICreateWorkItemView view,
            ITfsServerRepository tfsServerRepository, ITabsService tabsService, IUserSettingsService userSettingsService)
        {
            this._view = view;
            this.TfsServerRepository = tfsServerRepository;
            this.TabsService = tabsService;
            UserSettingsService = userSettingsService;
        }

        /// <summary>Initializes the presenter.</summary>
        /// <param name="project">A project to preselect options. Can be null.</param>
        public void Initialize(ProjectDescriptor project)
        {
            this._view.AttachPresenter(this);
            this._view.EnableCreate = false;
            this._view.DataSource = RetrieveServers();
            this.Project = project;

            if(project != null)
            {
                this._view.SelectServer(project.Server);
                this._view.SelectProject(project);

                if(String.IsNullOrEmpty(project.Configuration.DefaultWorkItemType) == false)
                {
                    var defaultType = project.WorkItemTypes.Find( workItemType => workItemType.Name.Equals(project.Configuration.DefaultWorkItemType));
                    this._view.SelectWorkItemType(defaultType);
                }
            }
        }

        public void Show()
        {
            this._view.ShowDialog();
        }

        public void HandleCreate()
        {
            var type = this._view.GetSelectedType();
            var server = this._view.GetSelectedServer();

            this.Project.Configuration.DefaultWorkItemType = type.Name;
            this.UserSettingsService.SaveProjectConfiguration(this.Project);

            var tfsServerManager = this.TfsServerRepository.GetServer(server);
            var presenter = ObjectFactory.With(tfsServerManager).GetInstance<IStandaloneWorkItemPresenter>();
            
            presenter.Initialize(new WorkItem(type), server);
            this.TabsService.OpenTab(presenter);
            this._view.Close();
        }

        public void HandleRefresh()
        {
            this._view.DataSource = new List<ServerDescriptor>();
            this.Initialize(null);
            ActivateCreateButton();
        }

        public void HandleSelectedTypeChanged()
        {
            ActivateCreateButton();
        }

        private List<ServerDescriptor> RetrieveServers()
        {
            return new List<ServerDescriptor>(
                from server in this.TfsServerRepository.GetConnectedServers()
                select server.ServerDescriptor);
        }

        private void ActivateCreateButton()
        {
            this._view.EnableCreate = this._view.GetSelectedType() != null;
        }
    }
}