using System.Collections.Generic;
using Nuwanda.Common;
using System.Windows.Forms;
using Nuwanda.TfsSpotlight.Data;
using System;
using Nuwanda.TfsSpotlight.Shell.Services.Interfaces;
using Nuwanda.TfsSpotlight.TFS;
using StructureMap;

namespace Nuwanda.TfsSpotlight.Views.GoToWorkItem
{
    [Author("Bartomiej Legid")]
    [Pluggable("GoToWorkItemPresenter")]
    public class GoToWorkItemPresenter : IGoToWorkItemPresenter
    {
        protected readonly Dictionary<ServerDescriptor, ToolStripMenuItem> _registeredServers;
        private readonly Dictionary<ProjectDescriptor, ToolStripMenuItem> _registeredProjects;

        private IGoToWorkItemView View { get; set; }
        private IMenuService MenuService { get; set; }
        private ITfsServerRepository ServerRepository { get; set; }
        private IClipboardService ClipboardService { get; set; }

        public ServerDescriptor SelectedServer { get; private set; }
        public ProjectDescriptor SelectedProject { get; private set; }
        public string SearchString { get; private set; }


        public int WorkItemId
        {
            get { return GetWorkItemId(); }
        }

        public int ServersCount
        {
            get { return this._registeredServers.Count; }
        }

        private int GetWorkItemId()
        {
            int workItemId;
            Int32.TryParse(this.View.GetTextBoxText().Trim(), out workItemId);
            return workItemId;
        }

        public GoToWorkItemPresenter(
            IGoToWorkItemView view, IMenuService menuService,
            ITfsServerRepository tfsServerRepository, IClipboardService clipboardService)
        {
            this.View= view;
            this.MenuService = menuService;
            this.ServerRepository = tfsServerRepository;
            ClipboardService = clipboardService;

            this._registeredServers = new Dictionary<ServerDescriptor, ToolStripMenuItem>();
            this._registeredProjects = new Dictionary<ProjectDescriptor, ToolStripMenuItem>();
        }

        public void Initialize()
        {
            this.View.AttachPresenter(this);

            this.MenuService.RegisterCommandActivator(CommandsNames.GoToWorkItem)
                .Activator(this.ActivateGoToWorkItemCommad).Context(this).Commit();

            RegisterEvents();
        }

        protected void RegisterEvents()
        {
            this.ServerRepository.ServerConnected +=
                (sender, e) => this.AddServer(e.TfsServerManager.ServerDescriptor);
            this.ServerRepository.ServerDisconnected +=
                (sender, e) => this.RemoveServer(e.TfsServerManager.ServerDescriptor);
            this.ServerRepository.ServerProjectsLoaded +=
                (sender, e) => this.RefreshServerProjects(e.TfsServerManager.ServerDescriptor);
        }

        protected bool ActivateGoToWorkItemCommad()
        {
            return this._registeredServers.Count > 0;
        }

        public void HandleKeyDown(Keys key)
        {
            if (key == Keys.Enter)
            {
                this.HandleExecute();
            }
        }

        private bool ShowProjects
        {
            get
            {
                var workItemId = GetWorkItemId();
                return workItemId <= 0 && String.IsNullOrEmpty(this.View.GetTextBoxText().Trim()) == false;
            }
        }

        public void HandleDropDownOpening()
        {
            SetUpDropDown(ShowProjects == false);
        }

        private void SetUpDropDown(bool showServers)
        {
            this.View.ToolStripInvoker.RunInControlsThread(
                delegate
                {
                    this.View.ClearDropDownItems();
                    if (showServers)
                    {
                        foreach (var item in _registeredServers.Values)
                        {
                            this.View.AddDropDownItem(item);
                        }
                    }
                    else
                    {
                        foreach (var item in _registeredProjects.Values)
                        {
                            this.View.AddDropDownItem(item);
                        }
                    }
                });
        }

        public void HandleExecute()
        {
            if(ShowProjects)
            {
                HandleExecuteProject();
            }
            else
            {
                HandleExecuteServer();
            }
        }

        private void HandleExecuteProject()
        {
            if (this._registeredProjects.Count == 1)
            {
                var project = new List<ProjectDescriptor>(this._registeredProjects.Keys)[0];
                this.HandleExecuteForProject(project);
            }
            else if (this._registeredProjects.Count > 1)
            {
                this.View.ShowDropDown();
            }
        }

        private void HandleExecuteServer()
        {
            if (this.ServersCount == 1)
            {
                var server = new List<ServerDescriptor>(this._registeredServers.Keys)[0];
                this.HandleExecuteForServer(server);
            }
            else if (this.ServersCount > 1)
            {
                this.View.ShowDropDown();
            }
        }

        public void Focus()
        {
            this.View.SetFocusToWorkItemIdTextBox();
            try
            {
                var textClipboardContent = this.ClipboardService.GetText();
                this.View.SetContent(textClipboardContent);
            }
            catch(InvalidOperationException) {}
        }


        protected void HandleServerClicked(object sender, EventArgs e)
        {
            var menuItem = sender as ToolStripMenuItem;
            if (menuItem == null) return;

            var selected = menuItem.Tag as ServerDescriptor;
            if(selected == null) return;

            this.View.HideDropDown();
            this.HandleExecuteForServer(selected);
        }

        private void HandleProjectClick(object sender, EventArgs e)
        {
            var menuItem = sender as ToolStripMenuItem;
            if (menuItem == null) return;

            var selected = menuItem.Tag as ProjectDescriptor;
            if (selected == null) return;

            this.View.HideDropDown();
            this.HandleExecuteForProject(selected);
        }

        private void HandleExecuteForServer(ServerDescriptor server)
        {
            this.SelectedServer = server;
            this.MenuService.RunCommand(CommandsNames.GoToWorkItem);
        }

        private void HandleExecuteForProject(ProjectDescriptor selected)
        {
            this.SelectedProject = selected;
            this.SearchString = this.View.GetTextBoxText().Trim();
            this.MenuService.RunCommand(CommandsNames.SearchWorkItemsByTitle);
        }

        protected void AddServer(ServerDescriptor server)
        {
            if (this._registeredServers.ContainsKey(server)) return;

            this.View.ToolStripInvoker.RunInControlsThread(
                delegate
                    {
                        var btn = new ToolStripMenuItem { Tag = server, Text = server.Name, ShowShortcutKeys = false, };
                        btn.Click += this.HandleServerClicked;

                        //this.View.AddDropDownItem(btn);
                        this._registeredServers.Add(server, btn);
                    });
            
            this.MenuService.InvalidateCommands(this);
        }

        protected void RemoveServer(ServerDescriptor server)
        {
            if(!this._registeredServers.ContainsKey(server)) return;

            //var btn = this._registeredServers[server];
            //this.View.RemoveDropDownItem(btn);
            this._registeredServers.Remove(server);
            foreach (var projectDescriptor in new List<ProjectDescriptor>(_registeredProjects.Keys))
            {
                if (projectDescriptor.Server.Equals(server))
                    this._registeredProjects.Remove(projectDescriptor);
            }

            this.MenuService.InvalidateCommands(this);
        }

        private void RefreshServerProjects(ServerDescriptor descriptor)
        {
            if (this._registeredServers.ContainsKey(descriptor) == false) return;

            //var serverButton = this._registeredServers[descriptor];

            this.View.ToolStripInvoker.RunInControlsThread(
                delegate
                {
                    //serverButton.DropDown.Items.Clear();

                    foreach (var project in descriptor.Projects)
                    {
                        var button = new ToolStripMenuItem { Tag = project, Text = String.Format("{0} ({1})", project.Name, descriptor.Name) , ShowShortcutKeys = false, };
                        button.Click += this.HandleProjectClick;

                        //serverButton.DropDown.Items.Add(button);
                        this._registeredProjects.Add(project, button);
                    }
                });
        }
    }
}