using System;
using System.Collections.Generic;
using System.Windows.Forms;
using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Services.Interfaces;
using Nuwanda.TfsSpotlight.Shell.Proxies.Interfaces;
using Nuwanda.TfsSpotlight.Shell.Services;
using Nuwanda.TfsSpotlight.Shell.Services.Interfaces;
using Nuwanda.TfsSpotlight.TFS;
using Nuwanda.TfsSpotlight.Views.Server;
using Nuwanda.TfsSpotlight.Views.Servers;
using StructureMap;
using Nuwanda.TfsSpotlight.Data;

namespace Nuwanda.TfsSpotlight.Views.Options
{
    [Author("Bartomiej Legid")]
    [Pluggable("OptionsPresenter")]
    public class OptionsPresenter : TabPresenterBase<IOptionsView>, IOptionsPresenter
    {
        private ApplicationSettings _dataSource;

        private IUserSettingsService UserSettingsService { get; set; }
        private IMonitService MonitService { get; set; }
        private ITfsServerRepository TfsServerRepository { get; set; }
        private IServersPresenter ServersPresenter { get; set; }
        private IObjectFactory ObjectFactory { get; set; }

        private readonly List<ServerDescriptor> _serversAdded;
        private readonly List<ServerDescriptor> _serversRemoved;

        public override bool IsDirty
        {
            get { return this._dataSource != null &&  this._dataSource.IsDirty; }
        }

        public OptionsPresenter(IOptionsView view, IWorker worker, IMenuService menuService, ILogger logger,
                                IUserSettingsService userSettingsService, IMonitService monitService,
                                ITfsServerRepository tfsServerRepository, IServersPresenter serversPresenter,
                                IObjectFactory objectFactory) 
            : base(view, worker, menuService, logger)
        {
            this.UserSettingsService = userSettingsService;
            this.MonitService = monitService;
            TfsServerRepository = tfsServerRepository;
            ServersPresenter = serversPresenter;
            ObjectFactory = objectFactory;

            this._serversRemoved = new List<ServerDescriptor>();
            this._serversAdded = new List<ServerDescriptor>();
        }

        public void Initialize()
        {
            this.View.AttachPresenter(this);
            this.View.Disable();

            this.MenuService.RegisterCommandActivator(CommandsNames.Save)
                .Activator(this.ActivateSaveCommand).Context(this).Commit();
            this.MenuService.RegisterCommandActivator(CommandsNames.Refresh)
                .Activator(this.ActivateRefreshCommand).Context(this).Commit();
        }

        private bool ActivateSaveCommand()
        {
            return DefaultCommandActivator() && this._dataSource != null && this._dataSource.IsDirty && this._dataSource.IsValid;
        }

        private bool ActivateRefreshCommand()
        {
            return DefaultCommandActivator();
        }

        public override void HandleClose()
        {
            if (!this.Worker.BeginWork()) return;

            try
            {
                if(this._dataSource != null && !this._dataSource.IsValid)
                {
                    this.MonitService.ShowStatusMessage(
                        String.Format("Invalid data from '{0}' tab was discarded.", Captions.TabSettingsNormal), MessageSeverity.Warning);
                }
                else if (this._dataSource != null && this._dataSource.IsDirty)
                {
                    var result = this.MonitService.ShowQuestion(
                        String.Format(Messages.QuestionSaveEditedWorkItemsWhileClosingQuery, Captions.TabSettingsNormal),
                        MessageBoxButtons.YesNo);

                    if (result == DialogResult.Yes)
                    {
                        this.Save();
                    }
                }
            }
            finally
            {
                this.Worker.EndWork();
            }
        }

        public override void HandleLoad()
        {
            if (!this.Worker.BeginWork()) return;

            try
            {
                this.Tab.SetCaption(Captions.TabSettingsLoading);
                this.InvalidateHandledCommands();
                this.View.Disable();

                Load();
                
                this.View.Enable();
            }
            catch (Exception e)
            {
                this.MonitService.ShowStatusMessage(String.Format(Messages.MsgInternalError, e.Message), MessageSeverity.Exception);
                this.Logger.WriteException(e);
            }
            finally
            {
                this.Worker.EndWork();
                this.Tab.RefreshCaption();
                this.InvalidateHandledCommands();
            }
        }

        private void Load()
        {
            this._dataSource = this.UserSettingsService.GetSettings();
            this._dataSource.PropertyChanged +=
                delegate
                    {
                        this.InvalidateHandledCommands();
                        this.Tab.RefreshCaption();
                        this.View.RefreshSelectedItem();
                    };

            this.View.DataSource = this._dataSource;
            this.View.ServersDataSource = this._dataSource.ServersSettings.Servers.GetSettingsItems();
        }

        public override void HandleRefresh()
        {
            if (!this.Worker.BeginWork()) return;

            try
            {
                this.Tab.SetCaption(Captions.TabSettingsRefreshing);
                this.InvalidateHandledCommands();
                this.View.Disable();
                
                Load();

                this.View.Enable();
            }
            catch (Exception e)
            {
                this.MonitService.ShowStatusMessage(String.Format(Messages.MsgInternalError, e.Message), MessageSeverity.Exception);
                this.Logger.WriteException(e);
            }
            finally
            {
                this.Worker.EndWork();
                this.Tab.RefreshCaption();
                this.InvalidateHandledCommands();
            }
        }

        public override string GetCaption()
        {
            return Captions.TabSettingsNormal;
        }

        public override void HandleSave()
        {
            if (!this.Worker.BeginWork()) return;

            try
            {
                this.InvalidateHandledCommands();
                this.View.Disable();
                this.Save();
            }
            catch (Exception e)
            {
                this.MonitService.ShowStatusMessage(String.Format(Messages.MsgInternalError, e.Message), MessageSeverity.Exception);
                this.Logger.WriteException(e);
            }
            finally
            {
                this.Worker.EndWork();
                this.View.Enable();
                this.Tab.RefreshCaption();
                this.InvalidateHandledCommands();
            }
        }

        private void Save()
        {
            this.UserSettingsService.SaveSettings(this._dataSource);
            var allServers = new List<ITfsServerManager>(this.TfsServerRepository.GetAllServers());

            foreach (var serverManager in allServers)
            {
                if(this._serversRemoved.Contains(serverManager.ServerDescriptor) == false) continue;

                serverManager.Disconnect();
                this.ServersPresenter.HandleServerRemoved(serverManager.ServerDescriptor);
            }

            foreach (var serverDescriptor in this._serversAdded)
            {
                var serverPresenter = this.ObjectFactory.GetInstance<IServerPresenter>();
                this.ServersPresenter.HandleServerAdded(serverPresenter);
                serverPresenter.Initialize(serverDescriptor);
            }

            this._serversAdded.Clear();
            this._serversRemoved.Clear();
        }

        public void HandleDiscoverServers()
        {
            foreach (var server in this.UserSettingsService.DiscoverServers())
            {
                if(this._dataSource.ServersSettings.Servers.Contains(server) == false)
                {
                    HandleAddServer(server);
                }
            }

            this.View.ServersDataSource = this._dataSource.ServersSettings.Servers.GetSettingsItems();
        }

        public void HandleAddServer()
        {
            HandleAddServer(new ServerDescriptor(String.Empty, "http://"));
        }

        public void HandleAddServer(ServerDescriptor serverDescriptor)
        {
            this._dataSource.ServersSettings.Servers.Add(serverDescriptor);
            var items = this._dataSource.ServersSettings.Servers.GetSettingsItems();
            this.View.ServersDataSource = items;
            this.View.SelectServer(items.Find(item => item.ServerDescriptor == serverDescriptor));

            if (this._serversRemoved.Contains(serverDescriptor))
            {
                this._serversRemoved.Remove(serverDescriptor);
            }
            else
            {
                this._serversAdded.Add(serverDescriptor);
            }
        }

        public void HandleDeleteServer()
        {
            var server = this.View.GetSelectedServer();
            if(server == null) return;

            this._dataSource.ServersSettings.Servers.Remove(server.ServerDescriptor);
            this.View.ServersDataSource = this._dataSource.ServersSettings.Servers.GetSettingsItems();
            
            if(this._serversAdded.Contains(server.ServerDescriptor))
            {
                this._serversAdded.Remove(server.ServerDescriptor);
            }
            else
            {
                this._serversRemoved.Add(server.ServerDescriptor);
            }
        }

        public override bool IsEqualTo(ITabPresenter presenter)
        {
            return presenter is IOptionsPresenter;
        }
    }
}