using System;
using System.Diagnostics;
using System.Threading;
using System.Windows.Forms;
using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Data;
using Nuwanda.TfsSpotlight.Services.Interfaces;
using Nuwanda.TfsSpotlight.Shell.Services.Interfaces;
using Nuwanda.TfsSpotlight.TFS;
using Nuwanda.TfsSpotlight.Views.Project;
using Nuwanda.TfsSpotlight.Views.Servers;
using StructureMap;

namespace Nuwanda.TfsSpotlight.Views.Server
{
    [Author("Bartomiej Legid")]
    [Pluggable("ServerPresenter")]
    public class ServerPresenter : IServerPresenter
    {
        public IServerView View { get; private set; }

        private ITfsServerManager _tfsServerManager;

        #region Command activators

        public bool CanConnect
        {
            get 
            { 
                return this._tfsServerManager.IsConnected == false && 
                    this._tfsServerManager.IsWorking == false &&
                    this._tfsServerManager.IsConnectionLost == false; 
            }
        }

        public bool CanDisconnect
        {
            get
            {
                return (this._tfsServerManager.IsConnected || this._tfsServerManager.IsConnectionLost) 
                    && this._tfsServerManager.IsWorking == false;
            }
        }

        public bool CanReconnect
        {
            get
            {
                return this._tfsServerManager.IsConnectionLost && this._tfsServerManager.IsWorking == false;
            }
        }

        public bool CanRefreshProjects
        {
            get
            {
                return this._tfsServerManager.IsConnected
                       && this._tfsServerManager.IsConnectionLost == false
                       && this._tfsServerManager.IsWorking == false;
            }
        }
        #endregion

        public bool IsConnected
        {
            get { return this._tfsServerManager.IsConnected; }
        }

        public event MethodInvoker ServerStatusChanged = delegate { };

        private ITfsServerRepository TfsServerRepository{ get; set; }
        private ITabsService TabsService { get; set; }
        private IObjectFactory ObjectFactory { get; set; }
        private IMenuService MenuService { get; set; }
        private IUserSettingsService UserSettingsService { get; set; }

        public ServerDescriptor ServeDescriptor { get; private set; }

        public bool IsConnectionLost 
        {
            get { return this._tfsServerManager.IsConnectionLost; }
        }

        public ServerPresenter(IServerView view, ITfsServerRepository tfsServerRepository,
            ITabsService tabsService, IObjectFactory objectFactory, IMenuService menuService, IUserSettingsService userSettingsService)
        {
            this.View = view;

            this.TfsServerRepository = tfsServerRepository;
            this.TabsService = tabsService;
            this.ObjectFactory = objectFactory;
            this.MenuService = menuService;
            UserSettingsService = userSettingsService;
        }

        public void Initialize(ServerDescriptor serverDescriptor)
        {
            this.ServeDescriptor = serverDescriptor;

            ConfigureServerManager();

            this.View.AttachPresenter(this);
            this.View.SetText(serverDescriptor.Name);

            this.View.ShowInfo("Not connected...", ServersNodeIcon.None);

            this.View.ContextMenu = this.MenuService.GetContextMenu(ContextMenusNames.Server);
        }

        private void ConfigureServerManager()
        {
            this._tfsServerManager = this.TfsServerRepository.GetServer(this.ServeDescriptor);
            this._tfsServerManager.Error += this.HandleError;
            this._tfsServerManager.Connecting += this.HandleConnecting;
            this._tfsServerManager.Connected += this.HandleConnected;
            this._tfsServerManager.Disconnecting += this.HandleDisconnecting;
            this._tfsServerManager.Disconnected += this.HandleDisconnected;
            this._tfsServerManager.RetrievingProjects += this.HandleRetrievingProjects;
            this._tfsServerManager.ProjectsRetrieved += this.HandleProjectsRetrieved;
            this._tfsServerManager.ConnectionLost += this.HandleConnectionLost;
            this._tfsServerManager.Reconnecting += this.HandleReconnecting;
            this._tfsServerManager.Reconnected += this.HandleReconnected;
        }

        private void HandleReconnected(object sender, EventArgs e)
        {
            this.View.HideInfo();
            this.View.SetIcon(ServersNodeIcon.ServerConnected);
            this.ServerStatusChanged();
            UnLockServerTabs();
        }

        private void HandleReconnecting(object sender, EventArgs e)
        {
            this.View.ShowInfo("Reconnecting...", ServersNodeIcon.Information);
            this.ServerStatusChanged();
        }

        private void HandleConnectionLost(object sender, EventArgs e)
        {
            this.View.ShowInfo("Connection lost. Try to reconnect.", ServersNodeIcon.Warning);
            this.View.SetIcon(ServersNodeIcon.ServerConnectionLost);
            this.ServerStatusChanged();
            LockServerTabs();
        }

        private void LockServerTabs()
        {
            var tabPresenters = this.TabsService.GetTabs(tab => tab is IServerAssociation &&
                                           (tab as IServerAssociation).IsAssociatedWith(this.ServeDescriptor));
            foreach (var list in tabPresenters)
            {
                list.HandleLock();
            }
        }

        private void UnLockServerTabs()
        {
            var tabPresenters = this.TabsService.GetTabs(tab => tab is IServerAssociation &&
                                           (tab as IServerAssociation).IsAssociatedWith(this.ServeDescriptor));
            foreach (var tabPresenter in tabPresenters)
            {
                tabPresenter.HandleUnLock();
            }
        }

        private void HandleConnecting(object sender, EventArgs e)
        {
            this.View.ShowInfo("Connecting...", ServersNodeIcon.Information);
            this.ServerStatusChanged();
        }

        private void HandleConnected(object sender, EventArgs e)
        {
            this.View.HideInfo();
            this.View.SetIcon(ServersNodeIcon.ServerConnected);
            this.ServerStatusChanged();
        }

        private void HandleDisconnecting(object sender, EventArgs e)
        {
            this.View.ClearProjects();
            this.View.ShowInfo("Disconnecting...", ServersNodeIcon.Information);
            this.ServerStatusChanged();
        }

        private void HandleDisconnected(object sender, EventArgs e)
        {
            this.View.ShowInfo("Not connected...", ServersNodeIcon.None);
            this.View.SetIcon(ServersNodeIcon.ServerNotConnected);
        }

        private void HandleError(object sender, ErrorEventArgs e)
        {
            this.View.ShowInfo(e.ErrorMessage, ServersNodeIcon.Error);
            this.ServerStatusChanged();
        }

        private void HandleRetrievingProjects(object sender, EventArgs e)
        {
            this.View.ClearProjects();
            this.View.ShowInfo("Retrieving projects...", ServersNodeIcon.Information);
            this.ServerStatusChanged();
        }

        private void HandleProjectsRetrieved(object sender, EventArgs e)
        {
            this.View.HideInfo();
        }

        public void HandleReconnect()
        {
            this._tfsServerManager.Reconnect();
        }

        public void HandleConnect()
        {
            this._tfsServerManager.Connect();
            if (this._tfsServerManager.IsConnected == false) return;

            HandleRefreshProjects();
        }

        public void HandleRefreshProjects()
        {
            var projects = this._tfsServerManager.GetProjects();

            foreach (var project in projects)
            {
                var projectPresenter = this.ObjectFactory.GetInstance<IProjectPresenter>();
                this.UserSettingsService.GetProjectConfiguration(project);
                var projectView = projectPresenter.Initialize(project);
                this.View.AddProject(projectView);
            }

            this.ServerStatusChanged();
        }


        public void HandleAutoConnect()
        {
            Debug.Assert(this.ServeDescriptor != null);

            if (this.ServeDescriptor.ConnectAtStartup)
                ThreadPool.QueueUserWorkItem(state => this.HandleConnect());
        }

        public void HandleDisconnect()
        {
            var tabPresenters = this.TabsService.GetTabs(
                tab => tab is IServerAssociation &&
                       (tab as IServerAssociation).IsAssociatedWith(this.ServeDescriptor));

            foreach (var tab in tabPresenters)
            {
                this.TabsService.CloseTab(tab);
            }

            this._tfsServerManager.Disconnect();
            this.View.ClearProjects();

            this.ServerStatusChanged();
        }
    }
}