using System.Windows.Forms;
using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Data;
using Nuwanda.TfsSpotlight.Shell.Services.Interfaces;
using Nuwanda.TfsSpotlight.Views.Server;
using StructureMap;

namespace Nuwanda.TfsSpotlight.Views.Servers
{
    [Author("Bartomiej Legid")]
    [Pluggable("ServersPresenter")]
    public class ServersPresenter : IServersPresenter
    {
        private readonly IServersView _view;
        private readonly IServersModel _model;

        private IMenuService MenuService { get; set; }

        public ServersPresenter(IServersModel model, IServersView view, IMenuService menuService)
        {
            this._model = model;
            this._view = view;
            this.MenuService = menuService;
        }

        public IServersView Initialize()
        {
            this._model.Load();
            this._view.AttachPresenter(this);
            this._view.DataSource = this._model.DataSource;

            this._model.DataSource.ForEach(presenter => presenter.ServerStatusChanged += delegate { this.InvalidateHandledCommands(); });

            this.MenuService.RegisterCommandActivator(CommandsNames.Connect)
                .Activator(this.ActivateConnectCommand).Context(this).Commit();
            this.MenuService.RegisterCommandActivator(CommandsNames.Disconnect)
                .Activator(this.ActivateDisconnectCommand).Context(this).Commit();
            this.MenuService.RegisterCommandActivator(CommandsNames.Reconnect)
                .Activator(this.ActivateReconnectCommand).Context(this).Commit();
            this.MenuService.RegisterCommandActivator(CommandsNames.RefreshProjects)
                .Activator(this.ActivateRefreshProjectsCommand).Context(this).Commit();

            return this._view;
        }

        private bool ActivateConnectCommand()
        {
            var server = this._view.GetSelectedServer();
            if(server == null) return false;

            return server.CanConnect;
        }

        private bool ActivateDisconnectCommand()
        {
            var server = this._view.GetSelectedServer();
            if (server == null) return false;

            return server.CanDisconnect;
        }

        private bool ActivateRefreshProjectsCommand()
        {
            var server = this._view.GetSelectedServer();
            if (server == null) return false;

            return server.CanRefreshProjects;
        }

        private bool ActivateReconnectCommand()
        {
            var server = this._view.GetSelectedServer();
            if (server == null) return false;

            return server.CanReconnect;
        }

        public void InvalidateHandledCommands()
        {
            this.MenuService.InvalidateCommands(this);
        }

        public Control ViewControl
        {
            get { return this._view as Control; }
        }

        public void HandleConnectSelected()
        {
            var server = this._view.GetSelectedServer();
            if (server == null) return;

            server.HandleConnect();
        }

        public void HandleAutoConnect()
        {
            if (this._model == null || this._model.DataSource == null)
                return;

            foreach (var serverPresenter in this._model.DataSource)
            {
                serverPresenter.HandleAutoConnect();
            }
        }

        public void HandleDoubleClick()
        {
            var selectedQuery = this._view.GetSelectedQuery();

            if(selectedQuery != null)
            {
                this.MenuService.RunCommand(CommandsNames.RunQuery);
                return;
            }

            var selectedBuild = this._view.GetSelectedBuild();
            if (selectedBuild != null)
            {
                this.MenuService.RunCommand(CommandsNames.OpenBuild);
                return;
            }
        }

        public void HandleSelectedNodeChanged()
        {
            this.MenuService.InvalidateCommands(this);
        }

        public void HandleDisconnectSelected()
        {
            var server = this._view.GetSelectedServer();
            if (server == null) return;

            server.HandleDisconnect();
        }

        public void HandleDisconnectAll()
        {
            foreach (var serverPresenter in this._model.DataSource)
            {
                serverPresenter.HandleDisconnect();
            }
        }

        public QueryDescriptor GetSelectedQuery()
        {
            return this._view.GetSelectedQuery();
        }

        public IServerPresenter GetSelectedServer()
        {
            return this._view.GetSelectedServer();
        }

        public BuildDescriptor GetSelectedBuild()
        {
            return this._view.GetSelectedBuild();
        }

        public ProjectDescriptor GetSelectedProject()
        {
            return this._view.GetSelectedProject();
        }

        public void HandleServerRemoved(ServerDescriptor serverDescriptor)
        {
            var serverPresenter = this._model.DataSource.Find(presenter => presenter.ServeDescriptor.Equals(serverDescriptor));
            if (serverPresenter == null) return;
            
            this._view.RemoveServerNode(serverPresenter.View as TreeNode);
        }

        public void HandleServerAdded(IServerPresenter serverPresenter)
        {
            this._model.DataSource.Add(serverPresenter);
            this._view.AddServerNode(serverPresenter.View as TreeNode);
        }
    }
}