using System;
using System.Collections.Generic;
using System.Windows.Forms;
using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Data;
using Nuwanda.TfsSpotlight.Views.Server;
using System.Linq;
using StructureMap;

namespace Nuwanda.TfsSpotlight.Views.Servers
{
    [Author("Bartomiej Legid")]
    [Pluggable("ServersView")]
    public class ServersView : TreeView, IServersView
    {
        private ServersPresenter _presenter;
        private ImageList _imageList;

        public IEnumerable<IServerPresenter> DataSource
        {
            set { SetDataSource(value); }
        }

        private void SetDataSource(IEnumerable<IServerPresenter> dataSource)
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new Action(() => this.SetDataSource(dataSource)));
                return;
            }

            this.Nodes.Clear();
            this.Nodes.AddRange(dataSource.Select(presenter => presenter.View).Cast<TreeNode>().ToArray());
        }

        public ServersView()
        {
            PrepareIcons();

            this.ImageList = this._imageList;
            this.BorderStyle = BorderStyle.None;
            this.FullRowSelect = true;
            this.HideSelection = false;
        }

        private void PrepareIcons()
        {
            this._imageList = new ImageList();
            this._imageList.Images.Add(Properties.Resources.TransparentIcon);
            this._imageList.Images.Add(Properties.Resources.ServerConnectedIcon);
            this._imageList.Images.Add(Properties.Resources.ServerNotConnectedIcon);
            this._imageList.Images.Add(Properties.Resources.ProjectIcon);
            this._imageList.Images.Add(Properties.Resources.QueryIcon);
            this._imageList.Images.Add(Properties.Resources.ErrorIcon);
            this._imageList.Images.Add(Properties.Resources.InformationIcon);
            this._imageList.Images.Add(Properties.Resources.WarningIcon);
            this._imageList.Images.Add(Properties.Resources.ServerConnectionLost);
            this._imageList.Images.Add(Properties.Resources.BuildsIcon);
            this._imageList.Images.Add(Properties.Resources.BuildTypeIcon);
        }

        public void AttachPresenter(ServersPresenter presenter)
        {
            this._presenter = presenter;
        }

        public IServerPresenter GetSelectedServer()
        {
            if (this.InvokeRequired)
            {
                return (IServerPresenter)this.Invoke(new Func<IServerPresenter>(this.GetSelectedServer));
            }

            var serverView = this.SelectedNode as IServerView;
            if (serverView == null) return null;
            return serverView.Presenter;
        }

        public QueryDescriptor GetSelectedQuery()
        {
            if (this.InvokeRequired)
            {
                return (QueryDescriptor)this.Invoke(new Func<QueryDescriptor>(this.GetSelectedQuery));
            }

            if (this.SelectedNode == null) return null;
            return this.SelectedNode.Tag as QueryDescriptor;
        }

        public BuildDescriptor GetSelectedBuild()
        {
            if (this.InvokeRequired)
            {
                return (BuildDescriptor)this.Invoke(new Func<BuildDescriptor>(this.GetSelectedBuild));
            }

            if (this.SelectedNode == null) return null;
            return this.SelectedNode.Tag as BuildDescriptor;
        }

        public ProjectDescriptor GetSelectedProject()
        {
            if (this.InvokeRequired)
            {
                return (ProjectDescriptor)this.Invoke(new Func<ProjectDescriptor>(this.GetSelectedProject));
            }

            if (this.SelectedNode == null) return null;
            return this.SelectedNode.Tag as ProjectDescriptor;
        }

        protected override void OnMouseDown(MouseEventArgs e)
        {
            var node = this.GetNodeAt(e.Location);
            if(node != null)
                this.SelectedNode = node;
            base.OnMouseDown(e);
        }

        protected override void OnMouseDoubleClick(MouseEventArgs e)
        {
            this._presenter.HandleDoubleClick();
            base.OnMouseDoubleClick(e);
        }

        protected override void OnAfterSelect(TreeViewEventArgs e)
        {
            base.OnAfterSelect(e);
            this._presenter.HandleSelectedNodeChanged();
        }

        public void RemoveServerNode(TreeNode serverNode)
        {
            if(this.InvokeRequired)
            {
                this.Invoke(new MethodInvoker(() => RemoveServerNode(serverNode)));
                return;
            }

            if(serverNode == null) return;

            this.Nodes.Remove(serverNode);
        }

        public void AddServerNode(TreeNode serverNode)
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new MethodInvoker(() => AddServerNode(serverNode)));
                return;
            }

            if (serverNode == null) return;

            this.Nodes.Add(serverNode);
        }
    }
}