using System.Net.Sockets;
using Microsoft.TeamFoundation.WorkItemTracking.Client;
using System;
using Nuwanda.TfsSpotlight.Data;
using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Shell.Services;
using Nuwanda.TfsSpotlight.Shell.Services.Interfaces;
using Nuwanda.TfsSpotlight.TFS;
using Nuwanda.TfsSpotlight.Views.Query;
using StructureMap;

namespace Nuwanda.TfsSpotlight.Views.WorkItems
{
    /// <summary>This presenter is used for work item's forms that are embedded into query tab</summary>
    [Author("Bartomiej Legid")]
    [Pluggable("EmbeddedWorkItemPresenter")]
    public class EmbeddedWorkItemPresenter : BaseWorkItemPresenter, IEmbeddedWorkItemPresenter
    {
        private IQueryPresenter _parent;

        public EmbeddedWorkItemPresenter(
            IWorkItemView view, IMonitService monitService, 
            IMenuService menuService, IWorker worker, ILogger logger,
            ITfsServerManager tfsServerManager) 
            : base(view, worker, monitService, menuService, logger, tfsServerManager)
        {}

        public void Initialize(ServerDescriptor server, IQueryPresenter parent)
        {
            this._server = server;
            this._parent = parent;
            this.View.SetCaption(String.Empty);

            this.MenuService.RegisterCommandActivator(CommandsNames.RefreshWorkItem)
                .Activator(this.ActivateRefreshCommand).Context(this).Commit();

            this.MenuService.RegisterCommandActivator(CommandsNames.Save)
                .Activator(this.ActivateSaveCommand).Context(this).Commit();
        }

        private bool ActivateRefreshCommand()
        {
            return this._parent.EmbeddedWorkItemCommandsAcivator() && this._dataSource != null;
        }

        private bool ActivateSaveCommand()
        {
            return this._parent.EmbeddedWorkItemCommandsAcivator() && this._dataSource != null && this._dataSource.IsDirty;
        }

        public void HandleLoad(WorkItem dataSource)
        {
            this.View.HideMessage();
            if (dataSource == null) return;
            
            this._dataSource = dataSource;

            this.View.SetCaption(String.Format(Captions.BarWorkItemLoading, this._dataSource.Title));
            this.MonitService.ShowStatusMessage(String.Format(Messages.MsgLoadingWorkItem, this._dataSource.Id, this._server.Name), MessageSeverity.Retrieving);
            this.View.Disable();

            this._dataSource.FieldChanged -= this.HandleWorkItemFieldChanged;
            this._dataSource.FieldChanged += this.HandleWorkItemFieldChanged;

            this.View.Enable();
            try
            {
                this.View.Invoker.RunInControlsThread(
                    delegate
                        {
                            var workItemControl = this.View.GetWorkItemControl();
                            workItemControl.SuspendLayout();
                            workItemControl.Item = this._dataSource;
                            workItemControl.ReadOnly = false;
                            workItemControl.ResumeLayout();
                        });
            }
            catch (SocketException se)
            {
                this.View.ShowError(String.Format(Messages.ExConnectionError, this.TfsServerManager.ServerDescriptor.Name));
                this.TfsServerManager.ReportConnectionLost();
                Logger.WriteException(se);
            }
            catch (ConnectionException ce)
            {
                this.View.ShowError(String.Format(Messages.ExConnectionError, this.TfsServerManager.ServerDescriptor.Name));
                this.TfsServerManager.ReportConnectionLost();
                Logger.WriteException(ce);
            }
            catch (DeniedOrNotExistException dae)
            {
                this.View.ShowError(String.Format(Messages.MsgWorkItemNotFound, this._dataSource.Id, this.TfsServerManager.ServerDescriptor.Name));
                Logger.WriteException(dae);
            }
            catch (Exception e)
            {
                this.View.ShowError(String.Format(Messages.MsgInternalError, e.Message));
                Logger.WriteException(e);
            }
            finally
            {
                this.MonitService.ClearStatusMessage();
                HandleWorkItemDataChanged();
            }
        }

        public override void HandleLoad()
        {
            //TODO: Move from query presenter
        }

        public override void HandleRefresh()
        {
            try
            {
                this.View.HideMessage();
                this.View.SetCaption(Captions.BarWorkItemRefreshingUnknown);
                this.MonitService.ShowStatusMessage(String.Format(Messages.MsgRefreshingWorkItem, this._dataSource.Title), MessageSeverity.Retrieving);
                this.View.Disable();

                this.TfsServerManager.Refresh(this._dataSource);

                this._dataSource.SyncToLatest();
                this.MonitService.ClearStatusMessage();
                HandleWorkItemDataChanged();
            }
            catch (BusinessLogicException ce)
            {
                this.View.SetCaption(String.Format(Captions.BarWorkItemUnableToRefresh, this._dataSource.Title));
                this.View.ShowError(ce.Message);
            }
            finally
            {
                this.View.Enable();
            }
        }

        protected override void HandleWorkItemDataChanged()
        {
            this.SetBarCaption();
            this.InvalidateHandledCommands();
            this._parent.HandleWorkItemDataChanged();
        }

        public void DisableView()
        {
            this.View.Disable();
        }

        public void EnableView()
        {
            this.View.Enable();
        }
    }
}