using System;
using System.Net.Sockets;
using System.Windows.Forms;
using Microsoft.TeamFoundation.WorkItemTracking.Client;
using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Services.Interfaces;
using Nuwanda.TfsSpotlight.Shell.Proxies.Interfaces;
using Nuwanda.TfsSpotlight.Shell.Services;
using Nuwanda.TfsSpotlight.Shell.Services.Interfaces;
using StructureMap;
using Nuwanda.TfsSpotlight.Data;
using Nuwanda.TfsSpotlight.TFS;

namespace Nuwanda.TfsSpotlight.Views.WorkItems
{
    /// <summary>This presenter is used for standalone work item's forms</summary>
    [Author("Bartomiej Legid")]
    [Pluggable("StandaloneWorkItemPresenter")]
    public class StandaloneWorkItemPresenter : BaseWorkItemPresenter, IStandaloneWorkItemPresenter
    {
        private IHistoryService HistoryService { get; set; }

        public int WorkItemId { get; private set; }

        public StandaloneWorkItemPresenter(
            IWorkItemView view, IMonitService monitService, IMenuService menuService,
            IWorker worker, ILogger logger,
            ITfsServerManager tfsServerManager, IHistoryService historyService) 
            : base(view, worker, monitService, menuService, logger, tfsServerManager)
        {
            this.HistoryService = historyService;
        }

        public void Initialize(int workItemId, ServerDescriptor server)
        {
            this._server = server;
            this.WorkItemId = workItemId;

            this.MenuService.RegisterCommandActivator(CommandsNames.Refresh)
                .Activator(this.ActivateRefreshCommand).Context(this).Commit();
            this.MenuService.RegisterCommandActivator(CommandsNames.Save)
                .Activator(this.ActivateSaveCommand).Context(this).Commit();
        }

        public void Initialize(WorkItem workItem, ServerDescriptor server)
        {
            this._dataSource = workItem;
            this.Initialize(-1, server);
        }

        #region Command activators

        private bool ActivateRefreshCommand()
        {
            return DefaultCommandActivator() && this.WorkItemId > 0;
        }

        private bool ActivateSaveCommand()
        {
            return DefaultCommandActivator() && this._dataSource != null && this._dataSource.IsDirty;
        }
        #endregion

        public override void HandleClose()
        {
            //TODO: Bug: Return true if closed, and TabService must take it into account!
            //     Close tab while working by using X on the right!
            if (!this.Worker.BeginWork()) return;

            try
            {
                if (this._dataSource != null && this._dataSource.IsDirty)
                {
                    this._dataSource.FieldChanged -= this.HandleWorkItemFieldChanged;

                    var result = this.MonitService.ShowQuestion(
                        String.Format(Messages.QuestionSaveEditedWorkItemsWhileClosingQuery, this.GetCaption()),
                        MessageBoxButtons.YesNo);

                    if (result == DialogResult.Yes)
                    {
                        this.HandleSave();
                    }
                }
            }
            finally
            {
                this.Worker.EndWork();
            }
        }

        public override void HandleLoad()
        {
            if (!this.Worker.BeginWork()) return;
            this.InvalidateHandledCommands();

            try
            {
                this.View.HideMessage();
                this.View.Disable();

                this.Tab.SetCaption(Captions.TabWorkItemLoading);
                this.View.SetCaption(Captions.BarWorkItemLoadingUnknown);
                if(this.WorkItemId > 0)
                    this.MonitService.ShowStatusMessage(String.Format(Messages.MsgLoadingWorkItem, this.WorkItemId, this._server.Name), MessageSeverity.Retrieving);
                else
                    this.MonitService.ShowStatusMessage(String.Format(Messages.MsgCreatingWorkItem, this._server.Name), MessageSeverity.Retrieving);
                
                this.MonitService.StartUnknownProgress();

                Load();
            }
            catch(BusinessLogicException ble)
            {
                this.View.ShowError(ble.Message);
            }
            catch (Exception e)
            {
                this.View.ShowError(String.Format(Messages.MsgInternalError, e.Message));
            }
            finally
            {
                this.MonitService.StopProgress();
                this.MonitService.ClearStatusMessage();
                this.View.Enable();

                this.Worker.EndWork();
                this.HandleWorkItemDataChanged();
                this.InvalidateHandledCommands();
            }
        }

        private void Load()
        {
            if(this.WorkItemId  > 0)
            {
                this._dataSource = this.TfsServerManager.GetWorkItem(this.WorkItemId);
            }

            this._dataSource.FieldChanged -= HandleWorkItemFieldChanged;
            this._dataSource.FieldChanged += HandleWorkItemFieldChanged;

            try
            {
                this.View.Invoker.RunInControlsThread(
                    delegate
                        {
                            var workItemControl = this.View.GetWorkItemControl();
                            workItemControl.SuspendLayout();
                            workItemControl.Item = this._dataSource;
                            workItemControl.ReadOnly = false;
                            workItemControl.ResumeLayout();
                        });
            }
            catch(SocketException se)
            {
                this.View.ShowError(String.Format(Messages.ExConnectionError, this.TfsServerManager.ServerDescriptor.Name));
                this.TfsServerManager.ReportConnectionLost();
                Logger.WriteException(se);    
            }
            catch (ConnectionException ce)
            {
                this.View.ShowError(String.Format(Messages.ExConnectionError, this.TfsServerManager.ServerDescriptor.Name));
                this.TfsServerManager.ReportConnectionLost();
                Logger.WriteException(ce);
            }
            catch (DeniedOrNotExistException dae)
            {
                this.View.ShowError(String.Format(Messages.MsgWorkItemNotFound, this._dataSource.Id, this.TfsServerManager.ServerDescriptor.Name));
                Logger.WriteException(dae);
            }
            catch (Exception e)
            {
                this.View.ShowError(String.Format(Messages.MsgInternalError, e.Message));
                Logger.WriteException(e);
            }
            finally
            {
                this.MonitService.ClearStatusMessage();

                if (this.WorkItemId > 0)
                    this.HistoryService.RegisterOperation(new HistoryEntry(this._dataSource.Id, this._dataSource.Title, this._server.Url, HistoryOperationType.View));
            }
        }
        
        public override void HandleRefresh()
        {
            if(!this.Worker.BeginWork()) return;
            this.InvalidateHandledCommands();

            try
            {
                this.View.HideMessage();
                this.View.Disable();

                this.Tab.SetCaption(Captions.TabWorkItemRefreshing);
                this.View.SetCaption(String.Format(Captions.BarWorkItemRefreshing, this._dataSource.Title));
                this.MonitService.ShowStatusMessage(String.Format(Messages.MsgRefreshingWorkItem, this._dataSource.Title), MessageSeverity.Retrieving);

                this.MonitService.StartUnknownProgress();

                Load();
            }
            catch (BusinessLogicException ce)
            {
                this.View.ShowError(ce.Message);
            }
            catch (Exception e)
            {
                this.View.ShowError(String.Format(Messages.MsgInternalError, e.Message));
            }
            finally
            {
                this.MonitService.StopProgress();
                this.MonitService.ClearStatusMessage();
                this.View.Enable();

                this.Worker.EndWork();
                this.HandleWorkItemDataChanged();
                this.InvalidateHandledCommands();
            }
        }

        public override string GetCaption()
        {
            if(this._dataSource == null)
            {
                return "No work item...";
            }

            if(this._dataSource.Id == 0)
            {
                return Captions.TabWorkItemNew;
            }
            
            return String.Format(Captions.TabWorkItemNormal, this._dataSource.Id);
        }

        public override void HandleSave()
        {
            if (!this.Worker.BeginWork()) return;
            this.InvalidateHandledCommands();

            try
            {
                this.View.HideMessage();
                base.HandleSave();
                this.WorkItemId = this._dataSource.Id;
            }
            finally
            {
                this.Worker.EndWork();
                this.InvalidateHandledCommands();
            }
        }

        protected override void HandleWorkItemDataChanged()
        {
            this.SetBarCaption();
            this.Tab.RefreshCaption();
        }

        public override bool IsEqualTo(ITabPresenter presenter)
        {
            if (presenter is StandaloneWorkItemPresenter)
            {
                var workItemPresenter = presenter as StandaloneWorkItemPresenter;
                return workItemPresenter._server.Equals(this._server) &&
                       (workItemPresenter.WorkItemId > 0 && this.WorkItemId > 0 &&
                        workItemPresenter.WorkItemId.Equals(this.WorkItemId)
                        ||
                        workItemPresenter._dataSource != null && this._dataSource != null &&
                        workItemPresenter._dataSource.Equals(this._dataSource));
            }

            return false;
        }

        public ProjectDescriptor Project
        {
            get
            {
                if (this._dataSource == null) return null;

                return this._server.Projects.Find(project => project.Name.Equals(this._dataSource.Project.Name));
            }
        }
    }
}