using System.Drawing;
using System.Windows.Forms;
using NUnit.Framework;
using Nuwanda.Common;
using Nuwanda.Windows.Forms;
using Rhino.Mocks;

namespace TfsSpotlight.Tests.Controls
{
    [Author("Bartomiej Legid")]
    [TestFixture]
    public class ListBoxTests
    {
        private MockRepository _mocks;
        private IListBoxItem _itemMock;

        private ListBoxTestableStub _testSubject;

        [SetUp]
        public void SetUp()
        {
            this._mocks = new MockRepository();
            this._itemMock = this._mocks.StrictMock<IListBoxItem>();

            this._testSubject = new ListBoxTestableStub();
        }

        [Test]
        public void WhenAskedTo_DrawItem_Should_DelegateDrawToItem()
        {
            DrawItemEventArgs e = new DrawItemEventArgs(null, null, Rectangle.Empty, 0, DrawItemState.Selected);

            using (this._mocks.Record())
            {
                this._itemMock.Draw(e);
            }

            using (this._mocks.Playback())
            {
                this._testSubject.Items.Add(this._itemMock);
                this._testSubject.Test_OnDrawItem(e);
            }
        }

        [Test]
        public void WhenAskedTo_MeasureItem_And_ReturnsNonZeroSize_Should_DelegateMeasurmentToItem_And_AssignReturnedValues()
        {
            MeasureItemEventArgs e = new MeasureItemEventArgs(null, 0);
            Size computedSize = new Size(100, 200);

            using (this._mocks.Record())
            {
                Expect.Call(this._itemMock.Measure(e)).Return(computedSize);
            }

            using (this._mocks.Playback())
            {
                this._testSubject.Items.Add(this._itemMock);
                this._testSubject.Test_OnMeasureItem(e);
                Assert.AreEqual(computedSize.Height, e.ItemHeight);
                Assert.AreEqual(computedSize.Width, e.ItemWidth);
            }
        }

        [Test]
        public void WhenAskedTo_MeasureItem_And_ReturnsZeroWidth_Should_DelegateMeasurmentToItem_And_AssignWidthAsClientWidth()
        {
            MeasureItemEventArgs e = new MeasureItemEventArgs(null, 0);
            Size computedSize = new Size(0, 200);

            using (this._mocks.Record())
            {
                Expect.Call(this._itemMock.Measure(e)).Return(computedSize);
            }

            using (this._mocks.Playback())
            {
                this._testSubject.Items.Add(this._itemMock);
                this._testSubject.Size = new Size(100, 200);
                this._testSubject.Test_OnMeasureItem(e);
                Assert.AreEqual(computedSize.Height, e.ItemHeight);
                Assert.AreEqual(this._testSubject.ClientRectangle.Width, e.ItemWidth);
            }
        }
    }
}
