using NUnit.Framework;
using Nuwanda.Common;
using Nuwanda.TfsSpotlight;
using Nuwanda.TfsSpotlight.Shell.Services.Interfaces;
using Nuwanda.TfsSpotlight.TFS;
using Nuwanda.TfsSpotlight.Views.GoToWorkItem;
using Rhino.Mocks;
using Rhino.Mocks.Constraints;
using Rhino.Mocks.Interfaces;
using System.Windows.Forms;
using System;

namespace TfsSpotlight.Tests.Views
{
    [Author("Bartomiej Legid")]
    [TestFixture]
    public class GoToWorkItemPresenterTests
    {
        private MockRepository _mocks;
        private IGoToWorkItemView _viewMock;
        private IMenuService _menuServiceMock;
        private ITfsServerRepository _tfsServerRepositoryMock;

        private GoToWorkItemPresenterTestableStub _testSubject;
        private IClipboardService _clipboardServiceMock;

        [SetUp]
        public void SetUp()
        {
            this._mocks = new MockRepository();
            this._viewMock = this._mocks.StrictMock<IGoToWorkItemView>();
            this._menuServiceMock = this._mocks.StrictMock<IMenuService>();
            this._tfsServerRepositoryMock = this._mocks.StrictMock<ITfsServerRepository>();
            this._clipboardServiceMock = this._mocks.StrictMock<IClipboardService>();

            this._testSubject = new GoToWorkItemPresenterTestableStub(
                this._viewMock, this._menuServiceMock, this._tfsServerRepositoryMock, this._clipboardServiceMock);
        }

        #region GetWorkItemId scrnarios

        [Test]
        public void WhenAskedFor_WorkItemId_And_WorkItemIdIsNotProvided_Should_ReturnZero()
        {
            using(this._mocks.Record())
            {
                Expect.Call(this._viewMock.GetTextBoxText()).Return(string.Empty);
            }

            using(this._mocks.Playback())
            {
                Assert.AreEqual(0, this._testSubject.WorkItemId);
            }
        }

        [Test]
        public void WhenAskedFor_WorkItemId_And_WorkItemIdIsWrong_Should_ReturnZero()
        {
            using (this._mocks.Record())
            {
                Expect.Call(this._viewMock.GetTextBoxText()).Return("asdasda");
            }

            using (this._mocks.Playback())
            {
                Assert.AreEqual(0, this._testSubject.WorkItemId);
            }
        }

        [Test]
        public void WhenAskedFor_WorkItemId_And_WorkItemIdIsCorrect_Should_ReturnProvidedWorkItemId()
        {
            using (this._mocks.Record())
            {
                Expect.Call(this._viewMock.GetTextBoxText()).Return("123");
            }

            using (this._mocks.Playback())
            {
                Assert.AreEqual(123, this._testSubject.WorkItemId);
            }
        }
        #endregion

        #region Initialize scenarios

        [Test]
        public void WhenAskedTo_Initialize_Should_AttachToView_And_RegisterCommandActvator_And_SubscribeForRepositoryEvents()
        {
            using (this._mocks.Record())
            {
                this._viewMock.AttachPresenter(this._testSubject);
                this._menuServiceMock.RegisterCommandActivator(CommandsNames.GoToWorkItem);
                //LastCall.Constraints(
                //    Is.Equal(this._testSubject),
                //    Is.Equal(CommandsNames.GoToWorkItem),
                //    Is.NotNull());
                Expect.Call(delegate { this._tfsServerRepositoryMock.ServerConnected += null; })
                    .Constraints(Is.NotNull());
                Expect.Call(delegate { this._tfsServerRepositoryMock.ServerDisconnected += null; })
                    .Constraints(Is.NotNull());
            }

            using (this._mocks.Playback())
            {
                this._testSubject.Initialize();
            }
        }
        #endregion

        #region AddRemove scenarios

        [Test]
        public void WhenAskedTo_AddServer_And_ThereIsAlreadySuchServer_Should_DoNothing()
        {
            var serverToAdd = ObjectMother.CreateNamedServerDescriptor();
            IEventRaiser raiser;
            var serverManagerMock = this._mocks.StrictMock<ITfsServerManager>();

            using (this._mocks.Record())
            {
                raiser = Expect.Call(delegate { this._tfsServerRepositoryMock.ServerConnected += null; }).IgnoreArguments().
                    GetEventRaiser();
                Expect.Call(delegate { this._tfsServerRepositoryMock.ServerDisconnected += null; })
                    .Constraints(Is.NotNull());
                Expect.Call(serverManagerMock.ServerDescriptor).Return(serverToAdd);
            }

            using (this._mocks.Playback())
            {
                this._testSubject.Test_AddServer(serverToAdd);
                this._testSubject.Test_SubscribeToEvents();
                raiser.Raise(this._tfsServerRepositoryMock, new ServerEventArgs(serverManagerMock));

                Assert.AreEqual(1, this._testSubject.ServersCount);
            }
        }

        [Test]
        public void WhenAskedTo_AddServer_And_ThereIsNoSuchServer_Should_AddGivenServer()
        {
            var serverToAdd = ObjectMother.CreateNamedServerDescriptor();
            ToolStripMenuItem createdMenuItem = null;
            IEventRaiser raiser;
            var serverManagerMock = this._mocks.StrictMock<ITfsServerManager>();

            using (this._mocks.Record())
            {
                raiser = Expect.Call(delegate { this._tfsServerRepositoryMock.ServerConnected += null; }).IgnoreArguments().
                    GetEventRaiser();
                Expect.Call(delegate { this._tfsServerRepositoryMock.ServerDisconnected += null; })
                    .Constraints(Is.NotNull());
                Expect.Call(serverManagerMock.ServerDescriptor).Return(serverToAdd);

                this._viewMock.AddDropDownItem(null);
                LastCall.IgnoreArguments()
                    .Do(new Action<ToolStripMenuItem>(item => createdMenuItem = item));
                this._menuServiceMock.InvalidateCommands(this._testSubject);
            }

            using (this._mocks.Playback())
            {
                this._testSubject.Test_SubscribeToEvents();
                raiser.Raise(this._tfsServerRepositoryMock, new ServerEventArgs(serverManagerMock));

                Assert.AreEqual(1, this._testSubject.ServersCount);
                Assert.IsNotNull(createdMenuItem);
                Assert.AreEqual(serverToAdd.Name, createdMenuItem.Text);
                Assert.AreSame(serverToAdd, createdMenuItem.Tag);
            }
        }

        [Test]
        public void WhenAskedTo_RemoveServer_And_ThereIsNoSuchServer_Should_DoNothing()
        {
            var serverToRemove = ObjectMother.CreateNamedServerDescriptor();
            IEventRaiser raiser;
            var serverManagerMock = this._mocks.StrictMock<ITfsServerManager>();

            using (this._mocks.Record())
            {
                raiser = Expect.Call(delegate { this._tfsServerRepositoryMock.ServerDisconnected += null; }).IgnoreArguments().
                    GetEventRaiser();
                Expect.Call(delegate { this._tfsServerRepositoryMock.ServerConnected += null; })
                    .Constraints(Is.NotNull());
                Expect.Call(serverManagerMock.ServerDescriptor).Return(serverToRemove);
            }

            using (this._mocks.Playback())
            {
                this._testSubject.Test_SubscribeToEvents();
                raiser.Raise(this._tfsServerRepositoryMock, new ServerEventArgs(serverManagerMock));

                Assert.AreEqual(0, this._testSubject.ServersCount);
            }
        }

        [Test]
        public void WhenAskedTo_RemoveServer_And_ThereIsSuchServer_Should_RemoveGivenServer()
        {
            var serverToRemove = ObjectMother.CreateNamedServerDescriptor();
            var associatedMenuItem = new ToolStripMenuItem();
            this._testSubject.Test_AddServer(serverToRemove, associatedMenuItem);
            IEventRaiser raiser;
            var serverManagerMock = this._mocks.StrictMock<ITfsServerManager>();

            using (this._mocks.Record())
            {
                raiser = Expect.Call(delegate { this._tfsServerRepositoryMock.ServerDisconnected += null; }).IgnoreArguments().
                    GetEventRaiser();
                Expect.Call(delegate { this._tfsServerRepositoryMock.ServerConnected += null; })
                    .Constraints(Is.NotNull());
                Expect.Call(serverManagerMock.ServerDescriptor).Return(serverToRemove);

                this._viewMock.RemoveDropDownItem(associatedMenuItem);
                this._menuServiceMock.InvalidateCommands(this._testSubject);
            }

            using (this._mocks.Playback())
            {
                this._testSubject.Test_SubscribeToEvents();
                raiser.Raise(this._tfsServerRepositoryMock, new ServerEventArgs(serverManagerMock));

                Assert.AreEqual(0, this._testSubject.ServersCount);
            }
        }
        #endregion

        #region Other scenarios

        [Test]
        public void WhenAskedTo_Focus_Should_ShouldFocusTextBox()
        {
            using (this._mocks.Record())
            {
                this._viewMock.SetFocusToWorkItemIdTextBox();
            }

            using (this._mocks.Playback())
            {
                this._testSubject.Focus();
            }
        }

        [Test]
        public void WhenAskedTo_ActivateGoToWorkItemCommad_And_ThereAreNoServers_Should_ReturnFalse()
        {
            Assert.IsFalse(this._testSubject.Test_ActivateGoToWorkItemCommad());
        }

        [Test]
        public void WhenAskedTo_ActivateGoToWorkItemCommad_And_ThereAreServers_Should_ReturnTrue()
        {
            this._testSubject.Test_AddServer(ObjectMother.CreateNamedServerDescriptor());
            Assert.IsTrue(this._testSubject.Test_ActivateGoToWorkItemCommad());
        }
        #endregion

        #region HandlingEnterKey scenarios

        [Test]
        public void WhenAskedTo_HandleInputKey_And_KeyIsEnter_And_ThereIsNoServer_Should_DoNothing()
        {
            using (this._mocks.Record())
            {
                //no expectations
            }

            using (this._mocks.Playback())
            {
                this._testSubject.HandleKeyDown(Keys.Enter);
            }
        }

        [Test]
        public void WhenAskedTo_HandleInputKey_And_KeyIsEnter_And_ThereIsMoreThanOneServer_Should_ShowDropDown()
        {
            this._testSubject.Test_AddServer(ObjectMother.CreateNamedServerDescriptor());
            this._testSubject.Test_AddServer(ObjectMother.CreateNamedServerDescriptor2());

            using (this._mocks.Record())
            {
                this._viewMock.ShowDropDown();
            }

            using (this._mocks.Playback())
            {
                this._testSubject.HandleKeyDown(Keys.Enter);
            }
        }

        [Test]
        public void WhenAskedTo_HandleInputKey_And_KeyIsEnter_And_ThereIsOneServer_Should_ExecuteCommandForServer()
        {
            var addedServer = ObjectMother.CreateNamedServerDescriptor();
            this._testSubject.Test_AddServer(addedServer);

            using (this._mocks.Record())
            {
                this._menuServiceMock.RunCommand(CommandsNames.GoToWorkItem);
            }

            using (this._mocks.Playback())
            {
                this._testSubject.HandleKeyDown(Keys.Enter);
                Assert.AreSame(addedServer, this._testSubject.SelectedServer);
            }
        }
        #endregion

        #region Server clicked scenarios

        [Test]
        public void When_ServerClicked_And_SenderIsNotToolStripMenuItem_Should_DoNothing()
        {
            this._mocks.ReplayAll();

            this._testSubject.Test_HandleServerClicked(new object(), EventArgs.Empty);

            this._mocks.VerifyAll();
        }

        [Test]
        public void When_ServerClicked_And_SenderTagIsNotServerDescriptor_Should_DoNothing()
        {
            this._mocks.ReplayAll();

            var menuItem = new ToolStripMenuItem {Tag = new object()};

            this._testSubject.Test_HandleServerClicked(menuItem, EventArgs.Empty);

            this._mocks.VerifyAll();
        }

        [Test]
        public void When_ServerClicked_And_HasAllRequiredParameters_Should_RunGoToWorkItemCommand()
        {
            var serverDescriptor = ObjectMother.CreateNamedServerDescriptor();
            var menuItem = new ToolStripMenuItem { Tag = serverDescriptor };

            using (this._mocks.Record())
            {
                this._viewMock.HideDropDown();
                this._menuServiceMock.RunCommand(CommandsNames.GoToWorkItem);
            }

            using (this._mocks.Playback())
            {
                this._testSubject.Test_HandleServerClicked(menuItem, EventArgs.Empty);
                Assert.AreSame(serverDescriptor, this._testSubject.SelectedServer);
            }
        }

        #endregion
    }
}