using System;
using NUnit.Framework;
using Nuwanda.Common;
using Nuwanda.TfsSpotlight.Services.Interfaces;
using Nuwanda.TfsSpotlight.Shell.Proxies.Interfaces;
using Nuwanda.TfsSpotlight.Shell.Services;
using Nuwanda.TfsSpotlight.Shell.Services.Interfaces;
using Nuwanda.TfsSpotlight.TFS;
using Nuwanda.TfsSpotlight.Views.Query;
using Nuwanda.TfsSpotlight.Views.WorkItems;
using Rhino.Mocks;
using Nuwanda.TfsSpotlight.Data;
using Rhino.Mocks.Constraints;
using Nuwanda.TfsSpotlight;
using Microsoft.TeamFoundation.WorkItemTracking.Client;

namespace TfsSpotlight.Tests.Views
{
    [TestFixture]
    [Author("Bartomiej Legid")]
    public class QueryPresenterTests
    {
        #region Setup

        private MockRepository _mocks;
        private IQueryView _viewMock;
        private IMenuService _menuServiceMock;
        private ITabsService _tabsServiceMock;
        private IMonitService _monitServiceMock;
        private IUserSettingsService _userSettingsServiceMock;
        private IWorker _workerMock;
        private ILogger _loggerMock;
        private ITabStripItem _tabMock;
        private IEmbeddedWorkItemPresenter _embeddedWorkItemPresenterMock;
        private ITfsServerRepository _tfsServerRepositoryMock;

        private QueryPresenterTestableStub _testSubject;
        private QueryDescriptor _query;
        private Guid _queryGuid;

        [SetUp]
        public void TestSetUp()
        {
            _mocks = new MockRepository();
            this._viewMock = this._mocks.DynamicMock<IQueryView>();
            this._menuServiceMock = this._mocks.StrictMock<IMenuService>();
            this._tabsServiceMock = this._mocks.DynamicMock<ITabsService>();
            this._monitServiceMock = this._mocks.DynamicMock<IMonitService>();
            this._userSettingsServiceMock = this._mocks.DynamicMock<IUserSettingsService>();
            this._workerMock = this._mocks.DynamicMock<IWorker>();
            this._loggerMock = this._mocks.DynamicMock<ILogger>();
            this._tabMock = this._mocks.DynamicMock<ITabStripItem>();
            this._embeddedWorkItemPresenterMock = this._mocks.DynamicMock<IEmbeddedWorkItemPresenter>();
            this._tfsServerRepositoryMock = this._mocks.StrictMock<ITfsServerRepository>();

            this._queryGuid = Guid.NewGuid();

            var server = ObjectMother.CreateNamedServerDescriptor();
            var project = new ProjectDescriptor("Project", server);
            this._query = new QueryDescriptor("QueryName", this._queryGuid, "WIQL", true, project);

            this._testSubject = new QueryPresenterTestableStub(
                this._viewMock, this._monitServiceMock,
                this._menuServiceMock, this._tabsServiceMock, this._userSettingsServiceMock,
                this._workerMock, this._loggerMock, this._embeddedWorkItemPresenterMock,
                this._tfsServerRepositoryMock) {Query = this._query, Tab = this._tabMock};
        }
        #endregion

        #region Refresh query scenarios

        [Test]
        public void WhenAskedTo_RefreshQuery_And_ThereIsAnotherCommandRunning_Should_NotPerformRefreshAction()
        {
            using (this._mocks.Record())
            {
                Expect.Call(this._workerMock.BeginWork()).Return(false);
                this._viewMock.DisableResultGrid();
                LastCall.Repeat.Never();
            }

            using (this._mocks.Playback())
            {
                this._testSubject.HandleRefresh();
            }
        }

        private void HelperSetUpPreRefreshQueryExpectations()
        {
            Expect.Call(this._workerMock.BeginWork()).Return(true);
            this._menuServiceMock.InvalidateCommands(this._testSubject);

            this._viewMock.SetQueryCaption(String.Empty, String.Empty);
            LastCall.Constraints(Is.Anything());

            this._embeddedWorkItemPresenterMock.DisableView();
            this._viewMock.DisableResultGrid();
            this._viewMock.StartResultGridProgress();
            this._viewMock.ClearResults();
        }

        private void HelperSetUpPostRefreshQueryExpectations()
        {
            this._viewMock.EnableResultGrid();
            this._embeddedWorkItemPresenterMock.EnableView();
            this._viewMock.StopResultGridProgress();
            this._workerMock.EndWork();
            this._menuServiceMock.InvalidateCommands(this._testSubject);
        }

        [Test]
        public void WhenAskedTo_RefreshQuery_And_ThereIsNoOtherCommandRunning_And_QueryResultWasNull_Should_RefreshQuery_But_StopAfterQueryRun_And_ShowProperInformation()
        {
            var tfsServerManager = this._mocks.StrictMock<ITfsServerManager>();

            using (this._mocks.Record())
            {
                HelperSetUpPreRefreshQueryExpectations();

                Expect.Call(tfsServerManager.RunQuery(this._query)).Return(null);

                this._monitServiceMock.ShowStatusMessage(Messages.MsgInternalErrorWhileRunningQuery, MessageSeverity.Error);

                HelperSetUpPostRefreshQueryExpectations();
            }

            using (this._mocks.Playback())
            {
                this._testSubject.Test_IsActive = true;
                this._testSubject.Test_ServerManager = tfsServerManager;
                this._testSubject.HandleRefresh();
            }
        }

        [Test]
        public void WhenAskedTo_RefreshQuery_And_ThereIsNoOtherCommandRunning_And_ThereIsQueryResult_Should_RefreshQuery_And_DisplayItToTheUser()
        {
            var tfsServerManager = this._mocks.StrictMock<ITfsServerManager>();

            using (this._mocks.Record())
            {
                HelperSetUpPreRefreshQueryExpectations();

                var result = new QueryResult(null, null);
                Expect.Call(tfsServerManager.RunQuery(this._query)).Return(result);

                this._viewMock.DisplayResult(result);
                this._viewMock.SetQueryCaption(String.Empty, String.Empty);
                LastCall.Constraints(Is.Anything());
                this._viewMock.SelectFirstQueryResult();

                HelperSetUpPostRefreshQueryExpectations();
            }

            using (this._mocks.Playback())
            {
                this._testSubject.Test_IsActive = true;
                this._testSubject.Test_ServerManager = tfsServerManager;
                this._testSubject.HandleRefresh();
            }
        }

        [Test]
        public void WhenAskedTo_Refresh_And_ThereIsNoOtherCommandRuning_And_ThereIsACouldNotConnectToTfsServerException_Should_NotifyTheUser()
        {
            var tfsServerManager = this._mocks.StrictMock<ITfsServerManager>();

            using (this._mocks.Record())
            {
                HelperSetUpPreRefreshQueryExpectations();

                Expect.Call(tfsServerManager.RunQuery(this._query)).Throw(new CouldNotConnectToTfsServerException());

                this._monitServiceMock.ShowStatusMessage(Messages.MsgCantConnectToServer, MessageSeverity.Error);
                this._viewMock.SetQueryCaption(String.Empty, String.Empty);
                LastCall.Constraints(Is.Anything());

                HelperSetUpPostRefreshQueryExpectations();
            }

            using (this._mocks.Playback())
            {
                this._testSubject.Test_IsActive = true;
                this._testSubject.Test_ServerManager = tfsServerManager;
                this._testSubject.HandleRefresh();
            }
        }

        [Test]
        public void WhenAskedTo_Refresh_And_ThereIsNoOtherCommandRuning_And_ThereIsAQueryValidationException_Should_NotifyTheUser_And_CloseTheTab()
        {
            var tfsServerManager = this._mocks.StrictMock<ITfsServerManager>();

            using (this._mocks.Record())
            {
                HelperSetUpPreRefreshQueryExpectations();

                Expect.Call(tfsServerManager.RunQuery(this._query)).Throw(new QueryValidationException());

                this._monitServiceMock.ShowStatusMessage(Messages.MsgInvalidQueryDefinition, MessageSeverity.Error);
                this._tabsServiceMock.CloseTab(this._testSubject);

                HelperSetUpPostRefreshQueryExpectations();
            }

            using (this._mocks.Playback())
            {
                this._testSubject.Test_IsActive = true;
                this._testSubject.Test_ServerManager = tfsServerManager;
                this._testSubject.HandleRefresh();
            }
        }
        #endregion

        #region Selected work item changed scenarios

        [Test]
        public void When_SelectedWorkItemChanged_Should_RunLoadWorkItemCommand()
        {
            using (this._mocks.Record())
            {
                this._menuServiceMock.RunCommand(CommandsNames.LoadWorkItem);
            }

            using (this._mocks.Playback())
            {
                this._testSubject.HandleSelectedWorkItemChanged();
            }
        }
        #endregion

        #region Work item load scenarios

        private void HelperSetUpPreLoadWorkItemExpectations()
        {
            this._workerMock.WaitAndBeginWork();
            this._menuServiceMock.InvalidateCommands(this._testSubject);
        }

        private void HelperSetUpPostLoadWorkItemExpectations()
        {
            this._tabMock.RefreshCaption();
            LastCall.IgnoreArguments();
            this._viewMock.UpdateResultGridRows();
            this._workerMock.EndWork();
            this._menuServiceMock.InvalidateCommands(this._testSubject);
        }

        [Test]
        public void WhenAskedTo_LoadWorkItem_And_ThereIsNoSelectedWorkItem_Should_HideWorkItemsDetails()
        {
            this._testSubject.Test_ReturnFromGetSelectedWorkItem(null);
            this._testSubject.Test_IsActive = true;

            using (this._mocks.Record())
            {
                HelperSetUpPreLoadWorkItemExpectations();

                this._viewMock.HideDetails();
                this._monitServiceMock.ClearStatusMessage();

                HelperSetUpPostLoadWorkItemExpectations();
            }

            using (this._mocks.Playback())
            {
                this._testSubject.HandleLoadWorkItem();
            }
        }

        [Test]
        public void WhenAskedTo_LoadWorkItem_And_ThereIsSelectedWorkItem_Should_DisplaySelectedWorkItemDetails()
        {
            var workItem = ObjectMother.GetUninitialized<WorkItem>();
            Assert.IsNotNull(workItem);

            this._testSubject.Test_IsActive = true;
            this._testSubject.Test_ReturnFromGetSelectedWorkItem(workItem);

            using (this._mocks.Record())
            {
                HelperSetUpPreLoadWorkItemExpectations();

                this._embeddedWorkItemPresenterMock.HandleActivation();
                this._viewMock.ShowDetails();
                this._embeddedWorkItemPresenterMock.HandleLoad(workItem);
                //this._viewMock.EnsureSelectedRowVisible();

                HelperSetUpPostLoadWorkItemExpectations();
            }

            using (this._mocks.Playback())
            {
                this._testSubject.HandleLoadWorkItem();
            }
        }
        #endregion
    }
}